@@ -8,18 +8,20 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CheatManager.cxx,v 1.13 2007/01/01 18:04:39 stephena Exp $
+// $Id: CheatManager.cxx 1873 2009-09-09 15:59:22Z stephena $
 //============================================================================
 
 #include <sstream>
 
 #include "OSystem.hxx"
+#include "Console.hxx"
 #include "Cheat.hxx"
+#include "Settings.hxx"
 #include "CheetahCheat.hxx"
 #include "BankRomCheat.hxx"
 #include "RamCheat.hxx"
@@ -53,7 +55,7 @@ const Cheat* CheatManager::add(const string& name, const string& code,
   // Delete duplicate entries
   for(unsigned int i = 0; i < myCheatList.size(); i++)
   {
-    if(myCheatList[i]->code() == code)
+    if(myCheatList[i]->name() == name || myCheatList[i]->code() == code)
     {
       myCheatList.remove_at(i);
       break;
@@ -230,7 +232,7 @@ void CheatManager::enable(const string& code, bool enable)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheatManager::loadCheatDatabase()
 {
-  string cheatfile = myOSystem->baseDir() + BSPF_PATH_SEPARATOR + "stella.cht";
+  const string& cheatfile = myOSystem->cheatFile();
   ifstream in(cheatfile.c_str(), ios::in);
   if(!in)
     return;
@@ -271,7 +273,7 @@ void CheatManager::saveCheatDatabase()
   if(!myListIsDirty)
     return;
 
-  string cheatfile = myOSystem->baseDir() + BSPF_PATH_SEPARATOR + "stella.cht";
+  const string& cheatfile = myOSystem->cheatFile();
   ofstream out(cheatfile.c_str(), ios::out);
   if(!out)
     return;
