@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundSDL.cxx,v 1.36 2007/01/01 18:04:40 stephena Exp $
+// $Id: SoundSDL.cxx 1862 2009-08-27 22:59:14Z stephena $
 //============================================================================
 
 #ifdef SOUND_SUPPORT
@@ -25,12 +25,16 @@
 
 #include "TIASnd.hxx"
 #include "FrameBuffer.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
 #include "Settings.hxx"
 #include "System.hxx"
 #include "OSystem.hxx"
 
+#include "Console.hxx"
+#include "AtariVox.hxx"
+#ifdef SPEAKJET_EMULATION
+  #include "SpeakJet.hxx"
+#endif
+
 #include "SoundSDL.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -39,7 +43,7 @@ SoundSDL::SoundSDL(OSystem* osystem)
     myIsEnabled(osystem->settings().getBool("sound")),
     myIsInitializedFlag(false),
     myLastRegisterSetCycle(0),
-    myDisplayFrameRate(60),
+    myDisplayFrameRate(60.0),
     myNumChannels(1),
     myFragmentSizeLogBase2(0),
     myIsMuted(false),
@@ -62,7 +66,7 @@ void SoundSDL::setEnabled(bool state)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL::initialize()
+void SoundSDL::open()
 {
   // Check whether to start the sound subsystem
   if(!myIsEnabled)
@@ -77,7 +81,7 @@ void SoundSDL::initialize()
   myRegWriteQueue.clear();
   myTIASound.reset();
 
-  if(!((SDL_WasInit(SDL_INIT_AUDIO) & SDL_INIT_AUDIO) > 0))
+  if(SDL_WasInit(SDL_INIT_AUDIO) == 0)
   {
     myIsInitializedFlag = false;
     myIsMuted = false;
@@ -96,17 +100,12 @@ void SoundSDL::initialize()
       Int32 tiafreq   = myOSystem->settings().getInt("tiafreq");
 
       SDL_AudioSpec desired;
-#ifndef PSP
       desired.freq   = frequency;
     #ifndef GP2X
       desired.format = AUDIO_U8;
     #else
       desired.format = AUDIO_U16;
     #endif
-#else
-      desired.freq   = 44100;
-      desired.format = AUDIO_U16;
-#endif
       desired.channels = myNumChannels;
       desired.samples  = fragsize;
       desired.callback = callback;
@@ -197,12 +196,6 @@ void SoundSDL::mute(bool state)
 {
   if(myIsInitializedFlag)
   {
-    // Ignore multiple calls to do the same thing
-    if(myIsMuted == state)
-    {
-      return;
-    }
-
     myIsMuted = state;
 
     SDL_PauseAudio(myIsMuted ? 1 : 0);
@@ -276,18 +269,14 @@ void SoundSDL::adjustCycleCounter(Int32 amount)
 void SoundSDL::setChannels(uInt32 channels)
 {
   if(channels == 1 || channels == 2)
-  {
     myNumChannels = channels;
-    myOSystem->settings().setInt("channels", myNumChannels);
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL::setFrameRate(uInt32 framerate)
+void SoundSDL::setFrameRate(float framerate)
 {
-  // FIXME, we should clear out the queue or adjust the values in it
+  // FIXME - should we clear out the queue or adjust the values in it?
   myDisplayFrameRate = framerate;
-  myLastRegisterSetCycle = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -304,7 +293,8 @@ void SoundSDL::set(uInt16 addr, uInt8 value, Int32 cycle)
   // the sound to "scale" correctly, we have to know the games real frame 
   // rate (e.g., 50 or 60) and the currently emulated frame rate. We use these
   // values to "scale" the time before the register change occurs.
-  delta = delta * (myDisplayFrameRate / (double)myOSystem->frameRate());
+// FIXME - this always results in 1.0, so we don't really need it
+//  delta = delta * (myDisplayFrameRate / myOSystem->frameRate());
   RegWrite info;
   info.addr = addr;
   info.value = value;
@@ -413,8 +403,9 @@ void SoundSDL::callback(void* udata, uInt8* stream, int len)
 {
   SoundSDL* sound = (SoundSDL*)udata;
   sound->processFragment(stream, (Int32)len);
-#ifdef ATARIVOX_SUPPORT
-  cerr << "SoundSDL::callback(): len==" << len << endl;
+
+#ifdef SPEAKJET_EMULATION
+//  cerr << "SoundSDL::callback(): len==" << len << endl;
 
   // See if we need sound from the AtariVox
   AtariVox *vox = sound->myOSystem->console().atariVox();
@@ -436,58 +427,9 @@ void SoundSDL::callback(void* udata, uInt8* stream, int len)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundSDL::load(Deserializer& in)
+bool SoundSDL::save(Serializer& out) const
 {
-  string device = "TIASound";
-
-  try
-  {
-    if(in.getString() != device)
-      return false;
-
-    uInt8 reg1 = 0, reg2 = 0, reg3 = 0, reg4 = 0, reg5 = 0, reg6 = 0;
-    reg1 = (uInt8) in.getInt();
-    reg2 = (uInt8) in.getInt();
-    reg3 = (uInt8) in.getInt();
-    reg4 = (uInt8) in.getInt();
-    reg5 = (uInt8) in.getInt();
-    reg6 = (uInt8) in.getInt();
-
-    myLastRegisterSetCycle = (Int32) in.getInt();
-
-    // Only update the TIA sound registers if sound is enabled
-    // Make sure to empty the queue of previous sound fragments
-    if(myIsInitializedFlag)
-    {
-      SDL_PauseAudio(1);
-      myRegWriteQueue.clear();
-      myTIASound.set(0x15, reg1);
-      myTIASound.set(0x16, reg2);
-      myTIASound.set(0x17, reg3);
-      myTIASound.set(0x18, reg4);
-      myTIASound.set(0x19, reg5);
-      myTIASound.set(0x1a, reg6);
-      SDL_PauseAudio(0);
-    }
-  }
-  catch(char *msg)
-  {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << device << endl;
-    return false;
-  }
-
-  return true;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundSDL::save(Serializer& out)
-{
-  string device = "TIASound";
+  const string& device = name();
 
   try
   {
@@ -506,23 +448,62 @@ bool SoundSDL::save(Serializer& out)
       reg6 = myTIASound.get(0x1a);
     }
 
-    out.putInt(reg1);
-    out.putInt(reg2);
-    out.putInt(reg3);
-    out.putInt(reg4);
-    out.putInt(reg5);
-    out.putInt(reg6);
+    out.putByte((char)reg1);
+    out.putByte((char)reg2);
+    out.putByte((char)reg3);
+    out.putByte((char)reg4);
+    out.putByte((char)reg5);
+    out.putByte((char)reg6);
 
     out.putInt(myLastRegisterSetCycle);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
-    cerr << msg << endl;
+    cerr << "ERROR: SoundSDL::save" << endl << "  " << msg << endl;
     return false;
   }
-  catch(...)
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool SoundSDL::load(Serializer& in)
+{
+  const string& device = name();
+
+  try
   {
-    cerr << "Unknown error in save state for " << device << endl;
+    if(in.getString() != device)
+      return false;
+
+    uInt8 reg1 = 0, reg2 = 0, reg3 = 0, reg4 = 0, reg5 = 0, reg6 = 0;
+    reg1 = (uInt8) in.getByte();
+    reg2 = (uInt8) in.getByte();
+    reg3 = (uInt8) in.getByte();
+    reg4 = (uInt8) in.getByte();
+    reg5 = (uInt8) in.getByte();
+    reg6 = (uInt8) in.getByte();
+
+    myLastRegisterSetCycle = (Int32) in.getInt();
+
+    // Only update the TIA sound registers if sound is enabled
+    // Make sure to empty the queue of previous sound fragments
+    if(myIsInitializedFlag)
+    {
+      SDL_PauseAudio(1);
+      myRegWriteQueue.clear();
+      myTIASound.set(0x15, reg1);
+      myTIASound.set(0x16, reg2);
+      myTIASound.set(0x17, reg3);
+      myTIASound.set(0x18, reg4);
+      myTIASound.set(0x19, reg5);
+      myTIASound.set(0x1a, reg6);
+      if(!myIsMuted) SDL_PauseAudio(0);
+    }
+  }
+  catch(const char* msg)
+  {
+    cerr << "ERROR: SoundSDL::load" << endl << "  " << msg << endl;
     return false;
   }
 
