@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerParser.cxx,v 1.96 2007/01/01 18:04:42 stephena Exp $
+// $Id: DebuggerParser.cxx 1874 2009-09-09 18:00:31Z stephena $
 //============================================================================
 
 #include <fstream>
@@ -23,10 +23,14 @@
 #include "Dialog.hxx"
 #include "Debugger.hxx"
 #include "CpuDebug.hxx"
+#include "RamDebug.hxx"
+#include "RiotDebug.hxx"
+#include "TIADebug.hxx"
 #include "DebuggerParser.hxx"
 #include "YaccParser.hxx"
 #include "M6502.hxx"
 #include "Expression.hxx"
+#include "FSNode.hxx"
 #include "RomWidget.hxx"
 
 #ifdef CHEATCODE_SUPPORT
@@ -122,16 +126,14 @@ string DebuggerParser::run(const string& command)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string DebuggerParser::exec(const string& file, bool verbose)
 {
-  string ret, path = file;
+  string ret;
   int count = 0;
   char buffer[256]; // FIXME: static buffers suck
 
-  if( file.find_last_of('.') == string::npos )
-    path += ".stella";
-
-  ifstream in(path.c_str());
+  FilesystemNode node(file);
+  ifstream in(node.getPath().c_str());
   if(!in.is_open())
-    return red("file \"" + path + "\" not found.");
+    return red("file \'" + node.getRelativePath() + "\' not found.");
 
   while( !in.eof() ) {
     if(!in.getline(buffer, 255))
@@ -149,7 +151,7 @@ string DebuggerParser::exec(const string& file, bool verbose)
   ret += "Executed ";
   ret += debugger->valueToString(count);
   ret += " commands from \"";
-  ret += path;
+  ret += file;
   ret += "\"\n";
   return ret;
 }
@@ -265,7 +267,7 @@ int DebuggerParser::decipher_arg(const string &str)
   else if(arg == "pc" || arg == ".") result = state.PC;
   else { // Not a special, must be a regular arg: check for label first
     const char *a = arg.c_str();
-    result = debugger->equateList->getAddress(arg);
+    result = debugger->equates().getAddress(arg);
 
     if(result < 0) { // if not label, must be a number
       if(bin) { // treat as binary
@@ -412,7 +414,6 @@ bool DebuggerParser::getArgs(const string& command, string& verb)
     argStrings.push_back(curArg);
 
   argCount = argStrings.size();
-
   /*
   cerr << "verb = " << verb << endl;
   cerr << "arguments (" << argCount << "):\n";
@@ -560,7 +561,8 @@ string DebuggerParser::eval()
   char buf[50];
   string ret;
   for(int i=0; i<argCount; i++) {
-    string label = debugger->equates()->getLabel(args[i]);
+    // TODO - technically, we should determine if the label is read or write
+    string label = debugger->equates().getLabel(args[i], true);
     if(label != "") {
       ret += label;
       ret += ": ";
@@ -599,7 +601,8 @@ string DebuggerParser::trapStatus(int addr)
   else
     result += "   none   ";
 
-  string l = debugger->equateList->getLabel(addr);
+  // TODO - technically, we should determine if the label is read or write
+  const string& l = debugger->equates().getLabel(addr, true);
   if(l != "") {
     result += "  (";
     result += l;
@@ -912,15 +915,6 @@ void DebuggerParser::executeExec()
   commandResult = exec(argStrings[0]);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// "height"
-void DebuggerParser::executeHeight()
-{
-  int height = debugger->setHeight(args[0]);
-  commandResult = "height set to " + debugger->valueToString(height, kBASE_10) +
-                  "\nExit debugger and reload ROM to take effect";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "help"
 void DebuggerParser::executeHelp()
@@ -936,6 +930,13 @@ void DebuggerParser::executeHelp()
   }
   commandResult += "\nBuilt-in functions:\n";
   commandResult += debugger->builtinHelp();
+  commandResult += "\nPseudo-registers:\n";
+  commandResult += "_scan     Current scanline count\n";
+  commandResult += "_bank     Currently selected bank\n";
+  commandResult += "_fcount   Number of frames since emulation started\n";
+  commandResult += "_cclocks  Color clocks on current scanline\n";
+  commandResult += "_vsync    Whether vertical sync is enabled (1 or 0)\n";
+  commandResult += "_vblank   Whether vertical blank is enabled (1 or 0)\n";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -973,6 +974,9 @@ void DebuggerParser::executeFunction()
 // "list"
 void DebuggerParser::executeList()
 {
+  if(!debugger->haveListFile())
+    commandResult = "no list file loaded (try \"loadlist file.lst\")";
+
   for(int i=args[0] - 2; i<args[0] + 3; i++)
     commandResult += debugger->getSourceLines(i);
 }
@@ -981,16 +985,19 @@ void DebuggerParser::executeList()
 // "listbreaks"
 void DebuggerParser::executeListbreaks()
 {
-  char buf[255];
+  ostringstream buf;
   int count = 0;
 
-  for(unsigned int i=0; i<0x10000; i++) {
-    if(debugger->breakPoints->isSet(i)) {
-      sprintf(buf, "%s ", debugger->equateList->getFormatted(i, 4));
-      commandResult += buf;
-      if(! (++count % 8) ) commandResult += "\n";
+  for(unsigned int i = 0; i < 0x10000; i++)
+  {
+    if(debugger->breakpoints().isSet(i))
+    {
+      buf << debugger->equates().getLabel(i, true, 4) << " ";
+      if(! (++count % 8) ) buf << "\n";
     }
   }
+  commandResult += buf.str();
+
   /*
   if(count)
     return ret;
@@ -1064,7 +1071,7 @@ void DebuggerParser::executeLoadlist()
 // "loadsym"
 void DebuggerParser::executeLoadsym()
 {
-  commandResult = debugger->equateList->loadFile(argStrings[0]);
+  commandResult = debugger->equates().loadFile(argStrings[0]);
   debugger->myRom->invalidate();
 }
 
@@ -1097,20 +1104,11 @@ void DebuggerParser::executePrint()
 void DebuggerParser::executeRam()
 {
   if(argCount == 0)
-    commandResult = debugger->dumpRAM();
+    commandResult = debugger->ramDebug().toString();
   else
     commandResult = debugger->setRAM(args);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// "reload"
-void DebuggerParser::executeReload()
-{
-  debugger->reloadROM();
-  debugger->start();
-  commandResult = "reloaded";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "reset"
 void DebuggerParser::executeReset()
@@ -1119,11 +1117,21 @@ void DebuggerParser::executeReset()
   commandResult = "reset CPU";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "rewind"
+void DebuggerParser::executeRewind()
+{
+  if(debugger->rewindState())
+    commandResult = "rewind by one level";
+  else
+    commandResult = "no states left to rewind";
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "riot"
 void DebuggerParser::executeRiot()
 {
-  commandResult = debugger->riotState();
+  commandResult = debugger->riotDebug().toString();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1156,7 +1164,7 @@ void DebuggerParser::executeRun()
 {
   debugger->saveOldState();
   debugger->quit();
-  commandResult = "exiting debugger";
+  commandResult = "_EXIT_DEBUGGER";  // See PromptWidget for more info
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1237,7 +1245,7 @@ void DebuggerParser::executeSavestate()
 // "savesym"
 void DebuggerParser::executeSavesym()
 {
-  if(debugger->equateList->saveFile(argStrings[0]))
+  if(debugger->equates().saveFile(argStrings[0]))
     commandResult = "saved symbols to file " + argStrings[0];
   else
     commandResult = red("I/O error");
@@ -1270,7 +1278,7 @@ void DebuggerParser::executeStep()
 // "tia"
 void DebuggerParser::executeTia()
 {
-  commandResult = debugger->dumpTIA();
+  commandResult = debugger->tiaDebug().toString();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1312,7 +1320,7 @@ void DebuggerParser::executeTrapwrite()
 // "undef"
 void DebuggerParser::executeUndef()
 {
-  if(debugger->equateList->undefine(argStrings[0]))
+  if(debugger->equates().removeEquate(argStrings[0]))
   {
     debugger->myRom->invalidate();
     commandResult = argStrings[0] + " now undefined";
@@ -1364,15 +1372,6 @@ void DebuggerParser::executeZ()
 }
 
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// .hxx
-// Table of RGB values for NTSC
-// static const uInt32 ourNTSCPalette[256];
-
-// .cxx
-//        const uInt32 Console::ourNTSCPalette[256] = {
-
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // List of all commands available to the parser
 DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
@@ -1556,15 +1555,6 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeFunction
   },
 
-  {
-    "height",
-    "Change height of debugger window",
-    true,
-    false,
-    { kARG_WORD, kARG_END_ARGS },
-    &DebuggerParser::executeHeight
-  },
-
   {
     "help",
     "This cruft",
@@ -1576,8 +1566,8 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
 
   {
     "list",
-    "List source (if loaded with loadlst)",
-    false,
+    "List source (if loaded with loadlist)",
+    true,
     false,
     { kARG_WORD, kARG_END_ARGS },
     &DebuggerParser::executeList
@@ -1682,15 +1672,6 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeRam
   },
 
-  {
-    "reload",
-    "Reload ROM and symbol file",
-    false,
-    true,
-    { kARG_END_ARGS },
-    &DebuggerParser::executeReload
-  },
-
   {
     "reset",
     "Reset 6507 to init vector (does not reset TIA, RIOT)",
@@ -1700,6 +1681,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeReset
   },
 
+  {
+    "rewind",
+    "Rewind state to last step/trace/scanline/frame advance",
+    false,
+    true,
+    { kARG_END_ARGS },
+    &DebuggerParser::executeRewind
+  },
+
   {
     "riot",
     "Show RIOT timer/input status",
@@ -1730,7 +1720,7 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
   {
     "runto",
     "Run until first occurrence of string in disassembly",
-    false,
+    true,
     true,
     { kARG_LABEL, kARG_END_ARGS },
     &DebuggerParser::executeRunTo
