@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIADebug.cxx,v 1.25 2007/01/01 18:04:42 stephena Exp $
+// $Id: TIADebug.cxx 1868 2009-09-01 11:54:59Z stephena $
 //============================================================================
 
 #include "System.hxx"
@@ -22,27 +22,26 @@
 #include "TIADebug.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-TIADebug::TIADebug(Debugger* dbg, Console* console)
+TIADebug::TIADebug(Debugger& dbg, Console& console)
   : DebuggerSystem(dbg, console),
-    mySystem(&console->system()),
-    myTIA((TIA*)&console->mediaSource())
+    myTIA(console.tia())
 {
-  nusizStrings[0] = "size=8 copy=1";
-  nusizStrings[1] = "size=8 copy=2 spac=8";
-  nusizStrings[2] = "size=8 copy=2 spac=$18";
-  nusizStrings[3] = "size=8 copy=3 spac=8";
-  nusizStrings[4] = "size=8 copy=2 spac=$38";
-  nusizStrings[5] = "size=$10 copy=1";
-  nusizStrings[6] = "size=8 copy=3 spac=$18";
-  nusizStrings[7] = "size=$20 copy=1";
+  nusizStrings[0] = "1 copy";
+  nusizStrings[1] = "2 copies - close (8)";
+  nusizStrings[2] = "2 copies - med (24)";
+  nusizStrings[3] = "3 copies - close (8)";
+  nusizStrings[4] = "2 copies - wide (56)";
+  nusizStrings[5] = "2x (16) sized player";
+  nusizStrings[6] = "3 copies - med (24)";
+  nusizStrings[7] = "4x (32) sized player";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-DebuggerState& TIADebug::getState()
+const DebuggerState& TIADebug::getState()
 {
   myState.ram.clear();
   for(int i = 0; i < 0x010; ++i)
-    myState.ram.push_back(myTIA->peek(i));
+    myState.ram.push_back(myTIA.peek(i));
 
   // Color registers
   myState.coluRegs.clear();
@@ -103,7 +102,7 @@ void TIADebug::saveOldState()
 {
   myOldState.ram.clear();
   for(int i = 0; i < 0x010; ++i)
-    myOldState.ram.push_back(myTIA->peek(i));
+    myOldState.ram.push_back(myTIA.peek(i));
 
   // Color registers
   myOldState.coluRegs.clear();
@@ -157,26 +156,26 @@ void TIADebug::saveOldState()
   myOldState.aud.push_back(audV1());
 }
 
-/* the set methods now use mySystem->poke(). This will save us the
-	trouble of masking the values here, since TIA::poke() will do it
-	for us.
+/* the set methods now use mySystem.poke(). This will save us the
+   trouble of masking the values here, since TIA::poke() will do it
+   for us.
 
-	This means that the GUI should *never* just display the value the
-	user entered: it should always read the return value of the set
-	method and display that.
+   This means that the GUI should *never* just display the value the
+   user entered: it should always read the return value of the set
+   method and display that.
 
-	An Example:
+   An Example:
 
-	User enters "ff" in the AUDV0 field. GUI calls value = tiaDebug->audV0(0xff).
-	The AUDV0 register is only 4 bits wide, so "value" is 0x0f. That's what
-	should be displayed.
+   User enters "ff" in the AUDV0 field. GUI calls value = tiaDebug->audV0(0xff).
+   The AUDV0 register is only 4 bits wide, so "value" is 0x0f. That's what
+   should be displayed.
 
-	In a perfect world, the GUI would only allow one hex digit to be entered...
-	but we allow decimal or binary input in the GUI (with # or \ prefix). The
-	only way to make that work would be to validate the data entry after every
-	keystroke... which would be a pain for both us and the user. Using poke()
-	here is a compromise that allows the TIA to do the range-checking for us,
-	so the GUI and/or TIADebug don't have to duplicate logic from TIA::poke().
+   In a perfect world, the GUI would only allow one hex digit to be entered...
+   but we allow decimal or binary input in the GUI (with # or \ prefix). The
+   only way to make that work would be to validate the data entry after every
+   keystroke... which would be a pain for both us and the user. Using poke()
+   here is a compromise that allows the TIA to do the range-checking for us,
+   so the GUI and/or TIADebug don't have to duplicate logic from TIA::poke().
 */
 
 //	 bool vdelP0(int newVal = -1);
@@ -187,90 +186,90 @@ void TIADebug::saveOldState()
 bool TIADebug::vdelP0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(VDELP0, ((bool)newVal));
+    mySystem.poke(VDELP0, ((bool)newVal));
 
-  return myTIA->myVDELP0;
+  return myTIA.myVDELP0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::vdelP1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(VDELP1, ((bool)newVal));
+    mySystem.poke(VDELP1, ((bool)newVal));
 
-  return myTIA->myVDELP1;
+  return myTIA.myVDELP1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::vdelBL(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(VDELBL, ((bool)newVal));
+    mySystem.poke(VDELBL, ((bool)newVal));
 
-  return myTIA->myVDELBL;
+  return myTIA.myVDELBL;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::enaM0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(ENAM0, ((bool)newVal) << 1);
+    mySystem.poke(ENAM0, ((bool)newVal) << 1);
 
-  return myTIA->myENAM0;
+  return myTIA.myENAM0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::enaM1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(ENAM1, ((bool)newVal) << 1);
+    mySystem.poke(ENAM1, ((bool)newVal) << 1);
 
-  return myTIA->myENAM1;
+  return myTIA.myENAM1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::enaBL(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(ENABL, ((bool)newVal) << 1);
+    mySystem.poke(ENABL, ((bool)newVal) << 1);
 
-  return myTIA->myENABL;
+  return myTIA.myENABL;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::resMP0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(RESMP0, ((bool)newVal) << 1);
+    mySystem.poke(RESMP0, ((bool)newVal) << 1);
 
-  return myTIA->myRESMP0;
+  return myTIA.myRESMP0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::resMP1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(RESMP1, ((bool)newVal) << 1);
+    mySystem.poke(RESMP1, ((bool)newVal) << 1);
 
-  return myTIA->myRESMP1;
+  return myTIA.myRESMP1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::refP0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(REFP0, ((bool)newVal) << 3);
+    mySystem.poke(REFP0, ((bool)newVal) << 3);
 
-  return myTIA->myREFP0;
+  return myTIA.myREFP0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::refP1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(REFP1, ((bool)newVal) << 3);
+    mySystem.poke(REFP1, ((bool)newVal) << 3);
 
-  return myTIA->myREFP1;
+  return myTIA.myREFP1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -278,15 +277,15 @@ bool TIADebug::refPF(int newVal)
 {
   if(newVal > -1)
   {
-    int tmp = myTIA->myCTRLPF;
+    int tmp = myTIA.myCTRLPF;
     if(newVal)
       tmp |= 0x01;
     else
       tmp &= ~0x01;
-    mySystem->poke(CTRLPF, tmp);
+    mySystem.poke(CTRLPF, tmp);
   }
 
-  return myTIA->myCTRLPF & 0x01;
+  return myTIA.myCTRLPF & 0x01;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -294,15 +293,15 @@ bool TIADebug::scorePF(int newVal)
 {
   if(newVal > -1)
   {
-    int tmp = myTIA->myCTRLPF;
+    int tmp = myTIA.myCTRLPF;
     if(newVal)
       tmp |= 0x02;
     else
       tmp &= ~0x02;
-    mySystem->poke(CTRLPF, tmp);
+    mySystem.poke(CTRLPF, tmp);
   }
 
-  return myTIA->myCTRLPF & 0x02;
+  return myTIA.myCTRLPF & 0x02;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -310,15 +309,15 @@ bool TIADebug::priorityPF(int newVal)
 {
   if(newVal > -1)
   {
-    int tmp = myTIA->myCTRLPF;
+    int tmp = myTIA.myCTRLPF;
     if(newVal)
       tmp |= 0x04;
     else
       tmp &= ~0x04;
-    mySystem->poke(CTRLPF, tmp);
+    mySystem.poke(CTRLPF, tmp);
   }
 
-  return myTIA->myCTRLPF & 0x04;
+  return myTIA.myCTRLPF & 0x04;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -329,147 +328,147 @@ bool TIADebug::collision(int collID, int newVal)
   if(newVal > -1)
   {
     if(newVal)
-      myTIA->myCollision |= mask;
+      myTIA.myCollision |= mask;
     else
-      myTIA->myCollision &= ~mask;
+      myTIA.myCollision &= ~mask;
   }
 
-  return myTIA->myCollision & mask;
+  return myTIA.myCollision & mask;
 }  
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::audC0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(AUDC0, newVal);
+    mySystem.poke(AUDC0, newVal);
 
-  return myTIA->myAUDC0;
+  return myTIA.myAUDC0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::audC1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(AUDC1, newVal);
+    mySystem.poke(AUDC1, newVal);
 
-  return myTIA->myAUDC1;
+  return myTIA.myAUDC1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::audV0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(AUDV0, newVal);
+    mySystem.poke(AUDV0, newVal);
 
-  return myTIA->myAUDV0;
+  return myTIA.myAUDV0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::audV1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(AUDV1, newVal);
+    mySystem.poke(AUDV1, newVal);
 
-  return myTIA->myAUDV1;
+  return myTIA.myAUDV1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::audF0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(AUDF0, newVal);
+    mySystem.poke(AUDF0, newVal);
 
-  return myTIA->myAUDF0;
+  return myTIA.myAUDF0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::audF1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(AUDF1, newVal);
+    mySystem.poke(AUDF1, newVal);
 
-  return myTIA->myAUDF1;
+  return myTIA.myAUDF1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::pf0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(PF0, newVal << 4);
+    mySystem.poke(PF0, newVal << 4);
 
-  return myTIA->myPF & 0x0f;
+  return myTIA.myPF & 0x0f;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::pf1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(PF1, newVal);
+    mySystem.poke(PF1, newVal);
 
-  return (myTIA->myPF & 0xff0) >> 4;
+  return (myTIA.myPF & 0xff0) >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::pf2(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(PF2, newVal);
+    mySystem.poke(PF2, newVal);
 
-  return (myTIA->myPF & 0xff000) >> 12;
+  return (myTIA.myPF & 0xff000) >> 12;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::coluP0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(COLUP0, newVal);
+    mySystem.poke(COLUP0, newVal);
 
-  return myTIA->myCOLUP0 & 0xff;
+  return myTIA.myColor[TIA::_P0] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::coluP1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(COLUP1, newVal);
+    mySystem.poke(COLUP1, newVal);
 
-  return myTIA->myCOLUP1 & 0xff;
+  return myTIA.myColor[TIA::_P1] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::coluPF(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(COLUPF, newVal);
+    mySystem.poke(COLUPF, newVal);
 
-  return myTIA->myCOLUPF & 0xff;
+  return myTIA.myColor[TIA::_PF] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::coluBK(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(COLUBK, newVal);
+    mySystem.poke(COLUBK, newVal);
 
-  return myTIA->myCOLUBK & 0xff;
+  return myTIA.myColor[TIA::_BK] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::nusiz0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(NUSIZ0, newVal);
+    mySystem.poke(NUSIZ0, newVal);
 
-  return myTIA->myNUSIZ0;
+  return myTIA.myNUSIZ0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::nusiz1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(NUSIZ1, newVal);
+    mySystem.poke(NUSIZ1, newVal);
 
-  return myTIA->myNUSIZ1;
+  return myTIA.myNUSIZ1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -477,12 +476,12 @@ uInt8 TIADebug::nusizP0(int newVal)
 {
   if(newVal > -1)
   {
-    uInt8 tmp = myTIA->myNUSIZ0 & ~0x07;
+    uInt8 tmp = myTIA.myNUSIZ0 & ~0x07;
     tmp |= (newVal & 0x07);
-    mySystem->poke(NUSIZ0, tmp);
+    mySystem.poke(NUSIZ0, tmp);
   }
 
-  return myTIA->myNUSIZ0 & 0x07;
+  return myTIA.myNUSIZ0 & 0x07;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -490,12 +489,12 @@ uInt8 TIADebug::nusizP1(int newVal)
 {
   if(newVal > -1)
   {
-    uInt8 tmp = myTIA->myNUSIZ1 & ~0x07;
+    uInt8 tmp = myTIA.myNUSIZ1 & ~0x07;
     tmp |= newVal & 0x07;
-    mySystem->poke(NUSIZ1, tmp);
+    mySystem.poke(NUSIZ1, tmp);
   }
 
-  return myTIA->myNUSIZ1 & 0x07;
+  return myTIA.myNUSIZ1 & 0x07;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -503,12 +502,12 @@ uInt8 TIADebug::nusizM0(int newVal)
 {
   if(newVal > -1)
   {
-    uInt8 tmp = myTIA->myNUSIZ0 & ~0x30;
+    uInt8 tmp = myTIA.myNUSIZ0 & ~0x30;
     tmp |= (newVal & 0x04) << 4;
-    mySystem->poke(NUSIZ0, tmp);
+    mySystem.poke(NUSIZ0, tmp);
   }
 
-  return (myTIA->myNUSIZ0 & 0x30) >> 4;
+  return (myTIA.myNUSIZ0 & 0x30) >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -516,85 +515,84 @@ uInt8 TIADebug::nusizM1(int newVal)
 {
   if(newVal > -1)
   {
-    uInt8 tmp = myTIA->myNUSIZ1 & ~0x30;
+    uInt8 tmp = myTIA.myNUSIZ1 & ~0x30;
     tmp |= (newVal & 0x04) << 4;
-    mySystem->poke(NUSIZ1, tmp);
+    mySystem.poke(NUSIZ1, tmp);
   }
 
-  return (myTIA->myNUSIZ1 & 0x30) >> 4;
+  return (myTIA.myNUSIZ1 & 0x30) >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::grP0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(GRP0, newVal);
+    mySystem.poke(GRP0, newVal);
 
-  return myTIA->myGRP0;
+  return myTIA.myGRP0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::grP1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(GRP1, newVal);
+    mySystem.poke(GRP1, newVal);
 
-  return myTIA->myGRP1;
+  return myTIA.myGRP1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posP0(int newVal)
 {
   if(newVal > -1)
-    myTIA->myPOSP0 = newVal;
-  return myTIA->myPOSP0;
+    myTIA.myPOSP0 = newVal;
+
+  return myTIA.myPOSP0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posP1(int newVal)
 {
   if(newVal > -1)
-    myTIA->myPOSP1 = newVal;
-  return myTIA->myPOSP1;
+    myTIA.myPOSP1 = newVal;
+
+  return myTIA.myPOSP1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posM0(int newVal)
 {
-/* FIXME
   if(newVal > -1)
-    mySystem->poke(???, newVal);
-*/
-  return myTIA->myPOSM0;
+    myTIA.myPOSM0 = newVal;
+
+  return myTIA.myPOSM0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posM1(int newVal)
 {
-/* FIXME
   if(newVal > -1)
-    mySystem->poke(???, newVal);
-*/
-  return myTIA->myPOSM1;
+    myTIA.myPOSM1 = newVal;
+
+  return myTIA.myPOSM1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posBL(int newVal)
 {
-/* FIXME
   if(newVal > -1)
-    mySystem->poke(???, newVal);
-*/
-  return myTIA->myPOSBL;
+    myTIA.myPOSBL = newVal;
+
+  return myTIA.myPOSBL;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::ctrlPF(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(CTRLPF, newVal);
+    mySystem.poke(CTRLPF, newVal);
 
-  return myTIA->myCTRLPF;
+  return myTIA.myCTRLPF;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -602,87 +600,87 @@ uInt8 TIADebug::sizeBL(int newVal)
 {
   if(newVal > -1)
   {
-    uInt8 tmp = myTIA->myCTRLPF & ~0x30;
+    uInt8 tmp = myTIA.myCTRLPF & ~0x30;
     tmp |= (newVal & 0x04) << 4;
-    mySystem->poke(CTRLPF, tmp);
+    mySystem.poke(CTRLPF, tmp);
   }
 
-  return (myTIA->myCTRLPF & 0x30) >> 4;
+  return (myTIA.myCTRLPF & 0x30) >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::hmP0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(HMP0, newVal << 4);
+    mySystem.poke(HMP0, newVal << 4);
 
-  return myTIA->myHMP0;
+  return myTIA.myHMP0 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::hmP1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(HMP1, newVal << 4);
+    mySystem.poke(HMP1, newVal << 4);
 
-  return myTIA->myHMP1;
+  return myTIA.myHMP1 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::hmM0(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(HMM0, newVal << 4);
+    mySystem.poke(HMM0, newVal << 4);
 
-  return myTIA->myHMM0;
+  return myTIA.myHMM0 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::hmM1(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(HMM1, newVal << 4);
+    mySystem.poke(HMM1, newVal << 4);
 
-  return myTIA->myHMM1;
+  return myTIA.myHMM1 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::hmBL(int newVal)
 {
   if(newVal > -1)
-    mySystem->poke(HMBL, newVal << 4);
+    mySystem.poke(HMBL, newVal << 4);
 
-  return myTIA->myHMBL;
+  return myTIA.myHMBL >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int TIADebug::frameCount()
 {
-  return myTIA->myFrameCounter;
+  return myTIA.myFrameCounter;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int TIADebug::scanlines()
 {
-  return myTIA->scanlines();
+  return myTIA.scanlines();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int TIADebug::clocksThisLine()
 {
-  return myTIA->clocksThisLine();
+  return myTIA.clocksThisLine();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::vsync()
 {
-  return (myTIA->myVSYNC & 2) == 2;
+  return (myTIA.myVSYNC & 2) == 2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::vblank()
 {
-  return (myTIA->myVBLANK & 2) == 2;
+  return (myTIA.myVBLANK & 2) == 2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -713,7 +711,7 @@ const string audFreq(uInt8 div)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string booleanWithLabel(string label, bool value)
+string TIADebug::booleanWithLabel(string label, bool value)
 {
   char buf[64];
   string ret;
@@ -733,20 +731,33 @@ const string booleanWithLabel(string label, bool value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string TIADebug::state()
+string TIADebug::toString()
 {
   string ret;
+  char buf[128];
+
+  sprintf(buf, "%.2x: ", 0);
+  ret += buf;
+  for (uInt8 j = 0; j < 0x010; j++)
+  {
+    sprintf(buf, "%.2x ", mySystem.peek(j));
+    ret += buf;
+
+    if(j == 0x07) ret += "- ";
+  }
+
+  ret += "\n";
 
   // TODO: inverse video for changed regs. Core needs to track this.
   // TODO: strobes? WSYNC RSYNC RESP0/1 RESM0/1 RESBL HMOVE HMCLR CXCLR
 
-  TiaState state    = (TiaState&) getState();
-// FIXME - change tracking  TiaState oldstate = (TiaState&) getOldState();
+  const TiaState& state    = (TiaState&) getState();
+//  const TiaState& oldstate = (TiaState&) getOldState();
 
   // build up output, then return it.
   ret += "scanline ";
 
-  ret += myDebugger->valueToString(myTIA->scanlines());
+  ret += myDebugger.valueToString(myTIA.scanlines());
   ret += " ";
 
   ret += booleanWithLabel("vsync", vsync());
@@ -755,38 +766,38 @@ string TIADebug::state()
   ret += booleanWithLabel("vblank", vblank());
   ret += "\n";
 
-  ret += booleanWithLabel("inpt0", myTIA->peek(0x08) & 0x80);
+  ret += booleanWithLabel("inpt0", myTIA.peek(0x08) & 0x80);
   ret += " ";
-  ret += booleanWithLabel("inpt1", myTIA->peek(0x09) & 0x80);
+  ret += booleanWithLabel("inpt1", myTIA.peek(0x09) & 0x80);
   ret += " ";
-  ret += booleanWithLabel("inpt2", myTIA->peek(0x0a) & 0x80);
+  ret += booleanWithLabel("inpt2", myTIA.peek(0x0a) & 0x80);
   ret += " ";
-  ret += booleanWithLabel("inpt3", myTIA->peek(0x0b) & 0x80);
+  ret += booleanWithLabel("inpt3", myTIA.peek(0x0b) & 0x80);
   ret += " ";
-  ret += booleanWithLabel("inpt4", myTIA->peek(0x0c) & 0x80);
+  ret += booleanWithLabel("inpt4", myTIA.peek(0x0c) & 0x80);
   ret += " ";
-  ret += booleanWithLabel("inpt5", myTIA->peek(0x0d) & 0x80);
+  ret += booleanWithLabel("inpt5", myTIA.peek(0x0d) & 0x80);
   ret += " ";
-  ret += booleanWithLabel("dump_gnd_0123", myTIA->myDumpEnabled);
+  ret += booleanWithLabel("dump_gnd_0123", myTIA.myDumpEnabled);
   ret += "\n";
 
   ret += "COLUP0: ";
-  ret += myDebugger->valueToString(state.coluRegs[0]);
+  ret += myDebugger.valueToString(state.coluRegs[0]);
   ret += "/";
   ret += colorSwatch(state.coluRegs[0]);
 
   ret += "COLUP1: ";
-  ret += myDebugger->valueToString(state.coluRegs[1]);
+  ret += myDebugger.valueToString(state.coluRegs[1]);
   ret += "/";
   ret += colorSwatch(state.coluRegs[1]);
 
   ret += "COLUPF: ";
-  ret += myDebugger->valueToString(state.coluRegs[2]);
+  ret += myDebugger.valueToString(state.coluRegs[2]);
   ret += "/";
   ret += colorSwatch(state.coluRegs[2]);
 
   ret += "COLUBK: ";
-  ret += myDebugger->valueToString(state.coluRegs[3]);
+  ret += myDebugger.valueToString(state.coluRegs[3]);
   ret += "/";
   ret += colorSwatch(state.coluRegs[3]);
 
@@ -795,11 +806,11 @@ string TIADebug::state()
   ret += "P0: GR=";
   ret += Debugger::to_bin_8(state.gr[P0]);
   ret += "/";
-  ret += myDebugger->valueToString(state.gr[P0]);
+  ret += myDebugger.valueToString(state.gr[P0]);
   ret += " pos=";
-  ret += myDebugger->valueToString(state.pos[P0]);
+  ret += myDebugger.valueToString(state.pos[P0]);
   ret += " HM=";
-  ret += myDebugger->valueToString(state.hm[P0]);
+  ret += myDebugger.valueToString(state.hm[P0]);
   ret += " ";
   ret += nusizP0String();
   ret += " ";
@@ -811,11 +822,11 @@ string TIADebug::state()
   ret += "P1: GR=";
   ret += Debugger::to_bin_8(state.gr[P1]);
   ret += "/";
-  ret += myDebugger->valueToString(state.gr[P1]);
+  ret += myDebugger.valueToString(state.gr[P1]);
   ret += " pos=";
-  ret += myDebugger->valueToString(state.pos[P1]);
+  ret += myDebugger.valueToString(state.pos[P1]);
   ret += " HM=";
-  ret += myDebugger->valueToString(state.hm[P1]);
+  ret += myDebugger.valueToString(state.hm[P1]);
   ret += " ";
   ret += nusizP1String();
   ret += " ";
@@ -825,37 +836,37 @@ string TIADebug::state()
   ret += "\n";
 
   ret += "M0: ";
-  ret += (myTIA->myENAM0 ? " ENABLED" : "disabled");
+  ret += (myTIA.myENAM0 ? " ENABLED" : "disabled");
   ret += " pos=";
-  ret += myDebugger->valueToString(state.pos[M0]);
+  ret += myDebugger.valueToString(state.pos[M0]);
   ret += " HM=";
-  ret += myDebugger->valueToString(state.hm[M0]);
+  ret += myDebugger.valueToString(state.hm[M0]);
   ret += " size=";
-  ret += myDebugger->valueToString(state.size[M0]);
+  ret += myDebugger.valueToString(state.size[M0]);
   ret += " ";
   ret += booleanWithLabel("reset", resMP0());
   ret += "\n";
 
   ret += "M1: ";
-  ret += (myTIA->myENAM1 ? " ENABLED" : "disabled");
+  ret += (myTIA.myENAM1 ? " ENABLED" : "disabled");
   ret += " pos=";
-  ret += myDebugger->valueToString(state.pos[M1]);
+  ret += myDebugger.valueToString(state.pos[M1]);
   ret += " HM=";
-  ret += myDebugger->valueToString(state.hm[M1]);
+  ret += myDebugger.valueToString(state.hm[M1]);
   ret += " size=";
-  ret += myDebugger->valueToString(state.size[M1]);
+  ret += myDebugger.valueToString(state.size[M1]);
   ret += " ";
   ret += booleanWithLabel("reset", resMP1());
   ret += "\n";
 
   ret += "BL: ";
-  ret += (myTIA->myENABL ? " ENABLED" : "disabled");
+  ret += (myTIA.myENABL ? " ENABLED" : "disabled");
   ret += " pos=";
-  ret += myDebugger->valueToString(state.pos[BL]);
+  ret += myDebugger.valueToString(state.pos[BL]);
   ret += " HM=";
-  ret += myDebugger->valueToString(state.hm[BL]);
+  ret += myDebugger.valueToString(state.hm[BL]);
   ret += " size=";
-  ret += myDebugger->valueToString(state.size[BL]);
+  ret += myDebugger.valueToString(state.size[BL]);
   ret += " ";
   ret += booleanWithLabel("delay", vdelBL());
   ret += "\n";
@@ -863,15 +874,15 @@ string TIADebug::state()
   ret += "PF0: ";
   ret += Debugger::to_bin_8(state.pf[0]);
   ret += "/";
-  ret += myDebugger->valueToString(state.pf[0]);
+  ret += myDebugger.valueToString(state.pf[0]);
   ret += " PF1: ";
   ret += Debugger::to_bin_8(state.pf[1]);
   ret += "/";
-  ret += myDebugger->valueToString(state.pf[1]);
+  ret += myDebugger.valueToString(state.pf[1]);
   ret += " PF2: ";
   ret += Debugger::to_bin_8(state.pf[2]);
   ret += "/";
-  ret += myDebugger->valueToString(state.pf[2]);
+  ret += myDebugger.valueToString(state.pf[2]);
   ret += "\n     ";
   ret += booleanWithLabel("reflect",  refPF());
   ret += " ";
@@ -900,31 +911,31 @@ string TIADebug::state()
   ret += "\n";
 
   ret += "AUDF0: ";
-  ret += myDebugger->valueToString(myTIA->myAUDF0);
+  ret += myDebugger.valueToString(myTIA.myAUDF0);
   ret += "/";
-  ret += audFreq(myTIA->myAUDF0);
+  ret += audFreq(myTIA.myAUDF0);
   ret += " ";
 
   ret += "AUDC0: ";
-  ret += myDebugger->valueToString(myTIA->myAUDC0);
+  ret += myDebugger.valueToString(myTIA.myAUDC0);
   ret += " ";
 
   ret += "AUDV0: ";
-  ret += myDebugger->valueToString(myTIA->myAUDV0);
+  ret += myDebugger.valueToString(myTIA.myAUDV0);
   ret += "\n";
 
   ret += "AUDF1: ";
-  ret += myDebugger->valueToString(myTIA->myAUDF1);
+  ret += myDebugger.valueToString(myTIA.myAUDF1);
   ret += "/";
-  ret += audFreq(myTIA->myAUDF1);
+  ret += audFreq(myTIA.myAUDF1);
   ret += " ";
 
   ret += "AUDC1: ";
-  ret += myDebugger->valueToString(myTIA->myAUDC1);
+  ret += myDebugger.valueToString(myTIA.myAUDC1);
   ret += " ";
 
   ret += "AUDV1: ";
-  ret += myDebugger->valueToString(myTIA->myAUDV1);
+  ret += myDebugger.valueToString(myTIA.myAUDV1);
   //ret += "\n";
 
   // note: last "ret +=" line should not contain \n, caller will add.
