@@ -8,24 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AudioWidget.cxx,v 1.5 2007/01/01 18:04:42 stephena Exp $
+// $Id: AudioWidget.cxx 1724 2009-05-13 13:55:40Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
+#include "DataGridWidget.hxx"
 #include "FrameBuffer.hxx"
-#include "GuiUtils.hxx"
 #include "GuiObject.hxx"
+#include "OSystem.hxx"
 #include "TIADebug.hxx"
 #include "Widget.hxx"
-#include "DataGridWidget.hxx"
+
 #include "AudioWidget.hxx"
 
 // ID's for the various widgets
@@ -108,7 +108,7 @@ void AudioWidget::handleCommand(CommandSender* sender, int cmd, int data, int id
   int addr, value;
   string buf;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
 
   switch(cmd)
@@ -140,7 +140,7 @@ void AudioWidget::fillGrid()
   IntArray vlist;
   BoolArray blist, changed, grNew, grOld;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
   TiaState state    = (TiaState&) tia.getState();
   TiaState oldstate = (TiaState&) tia.getOldState();
