@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DataGridWidget.cxx,v 1.10 2007/01/01 18:04:43 stephena Exp $
+// $Id: DataGridWidget.cxx 1755 2009-06-08 13:13:05Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -30,7 +30,8 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int cols, int rows,
-                               int colchars, int bits, BaseFormat base)
+                               int colchars, int bits, BaseFormat base,
+                               bool useScrollbar)
   : EditableWidget(boss, font, x, y,
                    cols*(colchars * font.getMaxCharWidth() + 8) + 1,
                    font.getLineHeight()*rows + 1),
@@ -43,7 +44,8 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
     _bits(bits),
     _base(base),
     _selectedItem(0),
-    _opsWidget(NULL)
+    _opsWidget(NULL),
+    _scrollBar(NULL)
 {
   _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
            WIDGET_WANTS_RAWDATA;
@@ -59,11 +61,22 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
   // Make sure hilite list contains all false values
   _hiliteList.clear();
   int size = _rows * _cols;
-  while((int)_hiliteList.size() < size)
+  while(size--)
     _hiliteList.push_back(false);
 
   // Set lower and upper bounds to sane values
   setRange(0, 1 << bits);
+
+  // Add a scrollbar if necessary
+  if(useScrollbar)
+  {
+    _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y, kScrollBarWidth, _h);
+    _scrollBar->setTarget(this);
+    _scrollBar->_numEntries = 1;
+    _scrollBar->_currentPos = 0;
+    _scrollBar->_entriesPerPage = 1;
+    _scrollBar->_wheel_lines = 1;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -90,15 +103,15 @@ cerr << "alist.size() = "     << alist.size()
   _valueStringList.clear();
   _changedList.clear();
 
-  _addrList     = alist;
-  _valueList    = vlist;
-  _changedList  = changed;
+  _addrList    = alist;
+  _valueList   = vlist;
+  _changedList = changed;
 
   // An efficiency thing
   string temp;
   for(int i = 0; i < size; ++i)
   {
-    temp = instance()->debugger().valueToString(_valueList[i], _base);
+    temp = instance().debugger().valueToString(_valueList[i], _base);
     _valueStringList.push_back(temp);
   }
 
@@ -118,9 +131,8 @@ cerr << "_addrList.size() = "     << _addrList.size()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DataGridWidget::setList(const int a, const int v, const bool c)
+void DataGridWidget::setList(int a, int v, bool c)
 {
-  // Convenience method for when the datagrid contains only one value
   IntArray alist, vlist;
   BoolArray changed;
 
@@ -132,46 +144,57 @@ void DataGridWidget::setList(const int a, const int v, const bool c)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DataGridWidget::setHiliteList(const IntArray& hilitelist)
+void DataGridWidget::setHiliteList(const BoolArray& hilitelist)
 {
-  // We can't assume this given list contains the exact number of
-  // items in this DataGrid, so we make sure
+  assert(hilitelist.size() == uInt32(_rows * _cols));
   _hiliteList.clear();
-  int size = _rows * _cols;
-  while((int)_hiliteList.size() < size)
-    _hiliteList.push_back(false);
-
-  // Now fill it with the addresses/positions given in 'hilitelist'
-  for(unsigned int i = 0; i < hilitelist.size(); ++i)
-  {
-    int pos = hilitelist[i];
-    if(pos >= 0 && pos <= size)
-      _hiliteList[pos] = true;
-  }
+  _hiliteList = hilitelist;
 
   setDirty(); draw();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setNumRows(int rows)
+{
+  if(_scrollBar)
+    _scrollBar->_numEntries = rows;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::setSelectedValue(int value)
 {
-  // Correctly format the data for viewing
-  _editString = instance()->debugger().valueToString(value, _base);
+  setValue(_selectedItem, value, _valueList[_selectedItem] != value);
+}
 
-  _valueStringList[_selectedItem] = _editString;
-  _changedList[_selectedItem] = (_valueList[_selectedItem] != value);
-  _valueList[_selectedItem] = value;
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setValue(int position, int value)
+{
+  setValue(position, value, _valueList[position] != value);
+}
 
-  sendCommand(kDGItemDataChangedCmd, _selectedItem, _id);
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setValue(int position, int value, bool changed)
+{
+  if(position >= 0 && uInt32(position) < _valueList.size())
+  {
+    // Correctly format the data for viewing
+    _editString = instance().debugger().valueToString(value, _base);
 
-  setDirty(); draw();
+    _valueStringList[position] = _editString;
+    _changedList[position] = changed;
+    _valueList[position] = value;
+
+    sendCommand(kDGItemDataChangedCmd, position, _id);
+
+    setDirty(); draw();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::setRange(int lower, int upper)
 {
-  _lowerBound = MAX(0, lower);
-  _upperBound = MIN(1 << _bits, upper);
+  _lowerBound = BSPF_max(0, lower);
+  _upperBound = BSPF_min(1 << _bits, upper);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -215,6 +238,12 @@ void DataGridWidget::handleMouseUp(int x, int y, int button, int clickCount)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::handleMouseWheel(int x, int y, int direction)
+{
+  _scrollBar->handleMouseWheel(x, y, direction);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int DataGridWidget::findItem(int x, int y)
 {
@@ -231,8 +260,8 @@ int DataGridWidget::findItem(int x, int y)
 bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
 {
   // Ignore all mod keys
-  if(instance()->eventHandler().kbdControl(modifiers) ||
-     instance()->eventHandler().kbdAlt(modifiers))
+  if(instance().eventHandler().kbdControl(modifiers) ||
+     instance().eventHandler().kbdAlt(modifiers))
     return true;
 
   bool handled = true;
@@ -317,7 +346,9 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         break;
 
       case 256+24:  // pageup
-        if (_currentRow > 0)
+        if(instance().eventHandler().kbdShift(modifiers) && _scrollBar)
+          handleMouseWheel(0, 0, -1);
+        else if (_currentRow > 0)
         {
           _currentRow = 0;
           dirty = true;
@@ -325,7 +356,9 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         break;
 
       case 256+25:	// pagedown
-        if (_currentRow < (int) _rows - 1)
+        if(instance().eventHandler().kbdShift(modifiers) && _scrollBar)
+          handleMouseWheel(0, 0, +1);
+        else if (_currentRow < (int) _rows - 1)
         {
           _currentRow = _rows - 1;
           dirty = true;
@@ -443,8 +476,8 @@ void DataGridWidget::handleCommand(CommandSender* sender, int cmd,
   switch (cmd)
   {
     case kSetPositionCmd:
-      if (_selectedItem != (int)data)
-        _selectedItem = data;
+      // Chain access; pass to parent
+      sendCommand(kSetPositionCmd, data, _id);
       break;
 
     case kDGZeroCmd:
@@ -481,17 +514,17 @@ void DataGridWidget::handleCommand(CommandSender* sender, int cmd,
 void DataGridWidget::drawWidget(bool hilite)
 {
 //cerr << "DataGridWidget::drawWidget\n";
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
   int row, col, deltax;
   string buffer;
 
   // Draw the internal grid and labels
   int linewidth = _cols * _colWidth;
   for (row = 0; row <= _rows; row++)
-    fb.hLine(_x, _y + (row * _rowHeight), _x + linewidth, kColor);
+    s.hLine(_x, _y + (row * _rowHeight), _x + linewidth, kColor);
   int lineheight = _rows * _rowHeight;
   for (col = 0; col <= _cols; col++)
-    fb.vLine(_x + (col * _colWidth), _y, _y + lineheight, kColor);
+    s.vLine(_x + (col * _colWidth), _y, _y + lineheight, kColor);
 
   // Draw the list items
   for (row = 0; row < _rows; row++)
@@ -505,7 +538,7 @@ void DataGridWidget::drawWidget(bool hilite)
       // Draw the selected item inverted, on a highlighted background.
       if (_currentRow == row && _currentCol == col &&
           _hasFocus && !_editMode)
-        fb.fillRect(x - 4, y - 2, _colWidth+1, _rowHeight+1, kTextColorHi);
+        s.fillRect(x - 4, y - 2, _colWidth+1, _rowHeight+1, kTextColorHi);
 
       if (_selectedItem == pos && _editMode)
       {
@@ -513,28 +546,28 @@ void DataGridWidget::drawWidget(bool hilite)
         adjustOffset();
         deltax = -_editScrollOffset;
 
-        fb.drawString(_font, buffer, x, y, _colWidth, kTextColor,
-                      kTextAlignLeft, deltax, false);
+        s.drawString(_font, buffer, x, y, _colWidth, kTextColor,
+                     kTextAlignLeft, deltax, false);
       }
       else
       {
         buffer = _valueStringList[pos];
         deltax = 0;
 
-        int color = kTextColor;
+        uInt32 color = kTextColor;
         if(_changedList[pos])
         {
-          fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kTextColorEm);
+          s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kDbgChangedColor);
 
           if(_hiliteList[pos])
-            color = kHiliteColor;
+            color = kDbgColorHi;
           else
-            color = kTextColorHi;
+            color = kDbgChangedTextColor;
         }
         else if(_hiliteList[pos])
-          color = kHiliteColor;
+          color = kDbgColorHi;
 
-        fb.drawString(_font, buffer, x, y, _colWidth, color);
+        s.drawString(_font, buffer, x, y, _colWidth, color);
       }
     }
   }
@@ -542,6 +575,10 @@ void DataGridWidget::drawWidget(bool hilite)
   // Only draw the caret while editing, and if it's in the current viewport
   if(_editMode)
     drawCaret();
+
+  // Draw the scrollbar
+  if(_scrollBar)
+    _scrollBar->recalc();  // takes care of the draw
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -578,7 +615,7 @@ void DataGridWidget::endEditMode()
   _editMode = false;
 
   // Update the both the string representation and the real data
-  int value = instance()->debugger().stringToValue(_editString);
+  int value = instance().debugger().stringToValue(_editString);
   if(value < _lowerBound || value >= _upperBound)
   {
     abortEditMode();
