@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerDialog.cxx,v 1.18 2007/01/01 18:04:43 stephena Exp $
+// $Id: DebuggerDialog.cxx 1860 2009-08-25 12:46:56Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -29,6 +29,7 @@
 #include "PromptWidget.hxx"
 #include "CpuWidget.hxx"
 #include "RamWidget.hxx"
+#include "RiotWidget.hxx"
 #include "RomWidget.hxx"
 #include "TiaWidget.hxx"
 #include "DataGridOpsWidget.hxx"
@@ -40,17 +41,18 @@
 #include "DebuggerDialog.hxx"
 
 enum {
-  kDDStepCmd  = 'DDst',
-  kDDTraceCmd = 'DDtr',
-  kDDAdvCmd   = 'DDav',
-  kDDSAdvCmd  = 'DDsv',
-  kDDExitCmd  = 'DDex'
+  kDDStepCmd   = 'DDst',
+  kDDTraceCmd  = 'DDtr',
+  kDDAdvCmd    = 'DDav',
+  kDDSAdvCmd   = 'DDsv',
+  kDDRewindCmd = 'DDrw',
+  kDDExitCmd   = 'DDex'
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DebuggerDialog::DebuggerDialog(OSystem* osystem, DialogContainer* parent,
                                int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h),
+  : Dialog(osystem, parent, x, y, w, h, true),  // use base surface
     myTab(NULL)
 {
   addTiaArea();
@@ -84,10 +86,10 @@ void DebuggerDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::handleKeyDown(int ascii, int keycode, int modifiers)
 {
-  bool handled = instance()->eventHandler().kbdAlt(modifiers);
+  bool handled = instance().eventHandler().kbdAlt(modifiers);
   if(handled)
   {
-    switch(ascii)
+    switch(keycode)
     {
       case 's':
         doStep();
@@ -101,6 +103,9 @@ void DebuggerDialog::handleKeyDown(int ascii, int keycode, int modifiers)
       case 'l':
         doScanlineAdvance();
         break;
+      case 'r':
+        doRewind();
+        break;
       default:
         handled = false;
         break;
@@ -134,6 +139,10 @@ void DebuggerDialog::handleCommand(CommandSender* sender, int cmd,
       doScanlineAdvance();
       break;
 
+    case kDDRewindCmd:
+      doRewind();
+      break;
+
     case kDDExitCmd:
       doExit();
       break;
@@ -146,21 +155,21 @@ void DebuggerDialog::handleCommand(CommandSender* sender, int cmd,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addTiaArea()
 {
-  GUI::Rect r = instance()->debugger().getTiaBounds();
+  GUI::Rect r = instance().debugger().getTiaBounds();
 
-  myTiaOutput = new TiaOutputWidget(this, instance()->consoleFont(),
+  myTiaOutput = new TiaOutputWidget(this, instance().consoleFont(),
                                     r.left, r.top, r.width(), r.height());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addTabArea()
 {
-  GUI::Rect r = instance()->debugger().getTabBounds();
+  GUI::Rect r = instance().debugger().getTabBounds();
 
   const int vBorder = 4;
 
   // The tab widget
-  myTab = new TabWidget(this, instance()->consoleFont(), r.left, r.top + vBorder,
+  myTab = new TabWidget(this, instance().consoleFont(), r.left, r.top + vBorder,
                         r.width(), r.height() - vBorder);
   addTabWidget(myTab);
 
@@ -170,94 +179,107 @@ void DebuggerDialog::addTabArea()
 
   // The Prompt/console tab
   tabID = myTab->addTab("Prompt");
-  myPrompt = new PromptWidget(myTab, instance()->consoleFont(),
+  myPrompt = new PromptWidget(myTab, instance().consoleFont(),
                               2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, myPrompt);
   addToFocusList(myPrompt->getFocusList(), tabID);
 
   // The TIA tab
   tabID = myTab->addTab("TIA");
-  TiaWidget* tia = new TiaWidget(myTab, instance()->consoleFont(),
+  TiaWidget* tia = new TiaWidget(myTab, instance().consoleFont(),
                                  2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, tia);
   addToFocusList(tia->getFocusList(), tabID);
 
+  // The input/output tab (includes RIOT and INPTx from TIA)
+  tabID = myTab->addTab("I/O");
+  RiotWidget* riot = new RiotWidget(myTab, instance().consoleFont(),
+                                    2, 2, widWidth, widHeight);
+  myTab->setParentWidget(tabID, riot);
+  addToFocusList(riot->getFocusList(), tabID);
+
   // The Audio tab
   tabID = myTab->addTab("Audio");
-  AudioWidget* aud = new AudioWidget(myTab, instance()->consoleFont(),
+  AudioWidget* aud = new AudioWidget(myTab, instance().consoleFont(),
                                      2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, aud);
   addToFocusList(aud->getFocusList(), tabID);
 
-  // The input/output tab (part of RIOT)
-//  tabID = myTab->addTab("I/O");
-
   myTab->setActiveTab(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addStatusArea()
 {
-  const GUI::Font& font = instance()->consoleFont();
-  GUI::Rect r = instance()->debugger().getStatusBounds();
+  const GUI::Font& font = instance().consoleFont();
+  const int lineHeight = font.getLineHeight();
+  GUI::Rect r = instance().debugger().getStatusBounds();
   int xpos, ypos;
 
   xpos = r.left;  ypos = r.top;
-  myTiaInfo = new TiaInfoWidget(this, instance()->consoleFont(), xpos, ypos);
+  myTiaInfo = new TiaInfoWidget(this, instance().consoleFont(), xpos, ypos);
 
   ypos += myTiaInfo->getHeight() + 10;
-  myTiaZoom = new TiaZoomWidget(this, instance()->consoleFont(), xpos+10, ypos);
+  myTiaZoom = new TiaZoomWidget(this, instance().consoleFont(), xpos+10, ypos,
+                                r.width()-10, r.height()-lineHeight-ypos-10);
   addToFocusList(myTiaZoom->getFocusList());
 
-  xpos += 10;  ypos += myTiaZoom->getHeight() + 20;
-  myMessageBox = new EditTextWidget(this, instance()->consoleFont(),
+  xpos += 10;  ypos += myTiaZoom->getHeight() + 10;
+  myMessageBox = new EditTextWidget(this, instance().consoleFont(),
                                     xpos, ypos, myTiaZoom->getWidth(),
                                     font.getLineHeight(), "");
   myMessageBox->setEditable(false);
   myMessageBox->clearFlags(WIDGET_RETAIN_FOCUS);
-  myMessageBox->setColor(kTextColorEm);
+  myMessageBox->setTextColor(kTextColorEm);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::addRomArea()
 {
-  GUI::Rect r = instance()->debugger().getRomBounds();
+  GUI::Rect r = instance().debugger().getRomBounds();
   int xpos, ypos;
 
   xpos = r.left + 10;  ypos = 10;
-  myCpu = new CpuWidget(this, instance()->consoleFont(), xpos, ypos);
+  myCpu = new CpuWidget(this, instance().consoleFont(), xpos, ypos);
   addToFocusList(myCpu->getFocusList());
 
   xpos = r.left + 10;  ypos += myCpu->getHeight() + 10;
-  myRam = new RamWidget(this, instance()->consoleFont(), xpos, ypos);
+  myRam = new RamWidget(this, instance().consoleFont(), xpos, ypos);
   addToFocusList(myRam->getFocusList());
 
   xpos = r.left + 10 + myCpu->getWidth() + 5;
-  DataGridOpsWidget* ops = new DataGridOpsWidget(this, instance()->consoleFont(),
+  DataGridOpsWidget* ops = new DataGridOpsWidget(this, instance().consoleFont(),
                                                  xpos, 20);
 
-  const int bwidth  = instance()->consoleFont().getStringWidth("Frame +1 "),
-            bheight = instance()->consoleFont().getLineHeight() + 2;
+  int bwidth  = instance().consoleFont().getStringWidth("Frame +1 "),
+      bheight = instance().consoleFont().getLineHeight() + 2;
   int buttonX = r.right - bwidth - 5, buttonY = r.top + 5;
-
-  new ButtonWidget(this, instance()->consoleFont(), buttonX, buttonY,
+  new ButtonWidget(this, instance().consoleFont(), buttonX, buttonY,
                    bwidth, bheight, "Step", kDDStepCmd);
   buttonY += bheight + 4;
-  new ButtonWidget(this, instance()->consoleFont(), buttonX, buttonY,
+  new ButtonWidget(this, instance().consoleFont(), buttonX, buttonY,
                    bwidth, bheight, "Trace", kDDTraceCmd);
   buttonY += bheight + 4;
-  new ButtonWidget(this, instance()->consoleFont(), buttonX, buttonY,
+  new ButtonWidget(this, instance().consoleFont(), buttonX, buttonY,
                    bwidth, bheight, "Scan +1", kDDSAdvCmd);
   buttonY += bheight + 4;
-  new ButtonWidget(this, instance()->consoleFont(), buttonX, buttonY,
+  new ButtonWidget(this, instance().consoleFont(), buttonX, buttonY,
                    bwidth, bheight, "Frame +1", kDDAdvCmd);
   buttonY += bheight + 4;
-  new ButtonWidget(this, instance()->consoleFont(), buttonX, buttonY,
+  new ButtonWidget(this, instance().consoleFont(), buttonX, buttonY,
                    bwidth, bheight, "Exit", kDDExitCmd);
-  buttonY += bheight + 4;
+
+  bwidth = instance().consoleFont().getStringWidth("< ") + 4;
+  bheight = bheight * 5 + 4*4;
+  buttonX -= (bwidth + 5);
+  buttonY = r.top + 5;
+  myRewindButton =
+    new ButtonWidget(this, instance().consoleFont(), buttonX, buttonY,
+                     bwidth, bheight, "<", kDDRewindCmd);
+  myRewindButton->clearFlags(WIDGET_ENABLED);
 
   xpos = r.left + 10;  ypos += myRam->getHeight() + 5;
-  myRom = new RomWidget(this, instance()->consoleFont(), xpos, ypos);
+  myRom = new RomWidget(this, instance().consoleFont(), xpos, ypos);
   addToFocusList(myRom->getFocusList());
 
   // Add the DataGridOpsWidget to any widgets which contain a
@@ -269,29 +291,35 @@ void DebuggerDialog::addRomArea()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doStep()
 {
-  instance()->debugger().parser()->run("step");
+  instance().debugger().parser().run("step");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doTrace()
 {
-  instance()->debugger().parser()->run("trace");
+  instance().debugger().parser().run("trace");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doAdvance()
 {
-  instance()->debugger().parser()->run("frame #1");
+  instance().debugger().parser().run("frame #1");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doScanlineAdvance()
 {
-  instance()->debugger().parser()->run("scanline #1");
+  instance().debugger().parser().run("scanline #1");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerDialog::doRewind()
+{
+  instance().debugger().parser().run("rewind");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doExit()
 {
-  instance()->debugger().quit();
+  instance().debugger().parser().run("run");
 }
