@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RomWidget.cxx,v 1.20 2007/01/01 18:04:44 stephena Exp $
+// $Id: RomWidget.cxx 1873 2009-09-09 15:59:22Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -33,10 +33,6 @@
 #include "RomListWidget.hxx"
 #include "RomWidget.hxx"
 
-enum {
-  kRomNameEntered = 'RWrn'
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   : Widget(boss, font, x, y, 16, 16),
@@ -47,8 +43,7 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
 {
   _type = kRomWidget;
 
-  int w = 58 * font.getMaxCharWidth(),
-      h = 0, xpos, ypos;
+  int xpos, ypos;
   StaticTextWidget* t;
 
   // Create bank editable area
@@ -60,10 +55,10 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
 
   xpos += t->getWidth() + 10;
   myBank = new DataGridWidget(boss, font, xpos, ypos-2,
-                              1, 1, 1, 2, kBASE_16_4);
+                              1, 1, 4, 8, kBASE_10);
   myBank->setTarget(this);
-  myBank->setRange(0, instance()->debugger().bankCount());
-  if(instance()->debugger().bankCount() <= 1)
+  myBank->setRange(0, instance().debugger().bankCount());
+  if(instance().debugger().bankCount() <= 1)
     myBank->setEditable(false);
   addFocusWidget(myBank);
 
@@ -76,16 +71,14 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
 
   xpos += t->getWidth() + 10;
   myBankCount = new EditTextWidget(boss, font, xpos, ypos-2,
-                                   20, font.getLineHeight(), "");
+                                   font.getStringWidth("XXXX"),
+                                   font.getLineHeight(), "");
   myBankCount->setEditable(false);
 
   // Create rom listing
   xpos = x;  ypos += myBank->getHeight() + 4;
-
-  // Update height of widget to use all remaining vertical space
-  GUI::Rect dialog = instance()->debugger().getDialogBounds();
-  int rows = ((dialog.height() - ypos) / font.getLineHeight()) - 1;
-  h = rows * font.getLineHeight();
+  GUI::Rect dialog = instance().debugger().getDialogBounds();
+  int w = dialog.width() - x - 5, h = dialog.height() - ypos - 3;
 
   myRomList = new RomListWidget(boss, font, xpos, ypos, w, h);
   myRomList->setTarget(this);
@@ -100,9 +93,8 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   // Create dialog box for save ROM (get name)
   StringList label;
   label.push_back("Filename: ");
-  mySaveRom = new InputTextDialog(boss, font, label, _x + 50, _y + 80);
+  mySaveRom = new InputTextDialog(boss, font, label);
   mySaveRom->setTarget(this);
-  mySaveRom->setCenter(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -110,6 +102,8 @@ RomWidget::~RomWidget()
 {
   myAddrList.clear();
   myLineList.clear();
+
+  delete mySaveRom;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -131,15 +125,15 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
     case kCMenuItemSelectedCmd:
     {
-      const string& rmb = myRomList->myMenu->getSelectedString();
+      const string& rmb = myRomList->myMenu->getSelectedTag();
 
-      if(rmb == "Save ROM")
+      if(rmb == "saverom")
       {
+        mySaveRom->show(_x + 50, _y + 80);
         mySaveRom->setTitle("");
         mySaveRom->setEmitSignal(kRomNameEntered);
-        parent()->addDialog(mySaveRom);
       }
-      else if(rmb == "Set PC")
+      else if(rmb == "setpc")
         setPC(myRomList->getSelected());
 
       break;
@@ -153,7 +147,7 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       else
       {
         saveROM(rom);
-        parent()->removeDialog();
+        parent().removeDialog();
       }
       break;
     }
@@ -161,7 +155,7 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
     case kDGItemDataChangedCmd:
     {
       int bank = myBank->getSelectedValue();
-      instance()->debugger().setBank(bank);
+      instance().debugger().setBank(bank);
     }
   }
 }
@@ -169,7 +163,7 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomWidget::loadConfig()
 {
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   bool bankChanged = myCurrentBank != dbg.getBank();
 
   // Only reload full bank when necessary
@@ -185,8 +179,9 @@ void RomWidget::loadConfig()
   myCurrentBank = dbg.getBank();
 
   // Update romlist to point to current PC
-  // Take mirroring of PC into account
-  int pc = dbg.cpuDebug().pc() | 0xe000;
+  // Take mirroring of PC into account, as well as zero-page RAM
+  int pc = dbg.cpuDebug().pc();
+  if(pc & 0x1000) pc |= 0xe000;
   AddrToLine::iterator iter = myLineList.find(pc);
 
   // if current PC not found, do an update (we're executing what
@@ -214,15 +209,14 @@ void RomWidget::loadConfig()
   myBank->setList(alist, vlist, changed);
 
   // Indicate total number of banks
-  int bankCount = dbg.bankCount();
-  myBankCount->setEditString(dbg.valueToString(bankCount));
+  myBankCount->setEditString(dbg.valueToString(dbg.bankCount(), kBASE_10));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomWidget::initialUpdate()
 {
-  Debugger& dbg = instance()->debugger();
-  PackedBitArray* bp = dbg.breakpoints();
+  Debugger& dbg = instance().debugger();
+  PackedBitArray& bp = dbg.breakpoints();
 
   // Reading from ROM might trigger a bankswitch, so save the current bank
   myCurrentBank = dbg.getBank();
@@ -239,11 +233,12 @@ void RomWidget::initialUpdate()
     StringList label, data, disasm;
     BoolArray state;
 
-    // Disassemble entire bank (up to 4096 lines) and reset breakpoints
-    dbg.disassemble(myAddrList, label, data, disasm, 0xf000, 4096);
+    // Disassemble zero-page RAM and entire bank and reset breakpoints
+//    dbg.disassemble(myAddrList, label, data, disasm, 0x80, 0xff);
+    dbg.disassemble(myAddrList, label, data, disasm, 0xf000, 0xffff);
     for(unsigned int i = 0; i < data.size(); ++i)
     {
-      if(bp && bp->isSet(myAddrList[i]))
+      if(bp.isSet(myAddrList[i]))
         state.push_back(true);
       else
         state.push_back(false);
@@ -271,7 +266,7 @@ void RomWidget::incrementalUpdate(int line, int rows)
 void RomWidget::setBreak(int data)
 {
   bool state = myRomList->getState(data);
-  instance()->debugger().setBreakPoint(myAddrList[data], state);
+  instance().debugger().setBreakPoint(myAddrList[data], state);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -279,7 +274,7 @@ void RomWidget::setPC(int data)
 {
   ostringstream command;
   command << "pc #" << myAddrList[data];
-  instance()->debugger().run(command.str());
+  instance().debugger().run(command.str());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -290,14 +285,14 @@ void RomWidget::patchROM(int data, const string& bytes)
   // Temporarily set to base 16, since that's the format the disassembled
   // byte string is in.  This eliminates the need to prefix each byte with
   // a '$' character
-  BaseFormat oldbase = instance()->debugger().parser()->base();
-  instance()->debugger().parser()->setBase(kBASE_16);
+  BaseFormat oldbase = instance().debugger().parser().base();
+  instance().debugger().parser().setBase(kBASE_16);
 
   command << "rom #" << myAddrList[data] << " " << bytes;
-  instance()->debugger().run(command.str());
+  instance().debugger().run(command.str());
 
   // Restore previous base
-  instance()->debugger().parser()->setBase(oldbase);
+  instance().debugger().parser().setBase(oldbase);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -305,5 +300,5 @@ void RomWidget::saveROM(const string& rom)
 {
   ostringstream command;
   command << "saverom " << rom;
-  instance()->debugger().run(command.str());
+  instance().debugger().run(command.str());
 }
