@@ -8,78 +8,30 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaWidget.cxx,v 1.7 2007/01/01 18:04:44 stephena Exp $
+// $Id: TiaWidget.cxx 1724 2009-05-13 13:55:40Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "FrameBuffer.hxx"
-#include "GuiUtils.hxx"
-#include "GuiObject.hxx"
-#include "TIADebug.hxx"
-#include "Widget.hxx"
-#include "EditTextWidget.hxx"
-#include "DataGridWidget.hxx"
 #include "ColorWidget.hxx"
+#include "DataGridWidget.hxx"
+#include "EditTextWidget.hxx"
+#include "FrameBuffer.hxx"
+#include "GuiObject.hxx"
+#include "OSystem.hxx"
+#include "TIADebug.hxx"
 #include "ToggleBitWidget.hxx"
 #include "TogglePixelWidget.hxx"
+#include "Widget.hxx"
+
 #include "TiaWidget.hxx"
 
-// ID's for the various widgets
-// We need ID's, since there are more than one of several types of widgets
-enum {
-  kP0_PFID,   kP0_BLID,   kP0_M1ID,   kP0_M0ID,   kP0_P1ID,
-  kP1_PFID,   kP1_BLID,   kP1_M1ID,   kP1_M0ID,
-  kM0_PFID,   kM0_BLID,   kM0_M1ID,
-  kM1_PFID,   kM1_BLID,
-  kBL_PFID,   // Make these first, since we want them to start from 0
-
-  kRamID,
-  kColorRegsID,
-  kGRP0ID,    kGRP1ID,
-  kPosP0ID,   kPosP1ID,
-  kPosM0ID,   kPosM1ID,   kPosBLID,
-  kHMP0ID,    kHMP1ID,
-  kHMM0ID,    kHMM1ID,    kHMBLID,
-  kRefP0ID,   kRefP1ID,
-  kDelP0ID,   kDelP1ID,   kDelBLID,
-  kNusizP0ID, kNusizP1ID,
-  kNusizM0ID, kNusizM1ID, kSizeBLID,
-  kEnaM0ID,   kEnaM1ID,   kEnaBLID,
-  kResMP0ID,  kResMP1ID,
-  kPF0ID,     kPF1ID,     kPF2ID,
-  kRefPFID,   kScorePFID, kPriorityPFID
-};
-
-// Strobe button commands
-enum {
-  kWsyncCmd = 'Swsy',
-  kRsyncCmd = 'Srsy',
-  kResP0Cmd = 'Srp0',
-  kResP1Cmd = 'Srp1',
-  kResM0Cmd = 'Srm0',
-  kResM1Cmd = 'Srm1',
-  kResBLCmd = 'Srbl',
-  kHmoveCmd = 'Shmv',
-  kHmclrCmd = 'Shmc',
-  kCxclrCmd = 'Scxl'
-};
-
-// Color registers
-enum {
-  kCOLUP0Addr,
-  kCOLUP1Addr,
-  kCOLUPFAddr,
-  kCOLUBKAddr
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
                      int x, int y, int w, int h)
@@ -91,56 +43,64 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight();
-  int xpos = 10, ypos = 25, lwidth = 4 * font.getMaxCharWidth();
+  int xpos = 10, ypos = 10, lwidth = 4 * font.getMaxCharWidth();
   StaticTextWidget* t;
 
   // Create a 16x1 grid holding byte values with labels
-  myRamGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos,
-                                 16, 1, 2, 8, kBASE_16);
-  myRamGrid->setEditable(false);
-  myRamGrid->setTarget(this);
-  myRamGrid->setID(kRamID);
-  addFocusWidget(myRamGrid);
-
-  t = new StaticTextWidget(boss, font, xpos, ypos + 2,
-                           lwidth-2, fontHeight,
-                           "00:", kTextAlignLeft);
-  for(int col = 0; col < 16; ++col)
+  // Only do so if we have the vertical height
+  // TODO - at some point, viewing RAM may be removed, as I don't know
+  //        how useful it really is
+  if(h >= 400)
   {
-    t = new StaticTextWidget(boss, font, xpos + col*myRamGrid->colWidth() + lwidth + 7,
-                             ypos - lineHeight,
-                             fontWidth, fontHeight,
-                             Debugger::to_hex_4(col),
-                             kTextAlignLeft);
-  }
+    ypos += lineHeight;
+    myRamGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos,
+                                   16, 1, 2, 8, kBASE_16);
+    myRamGrid->setEditable(false);
+    myRamGrid->setTarget(this);
+    myRamGrid->setID(kRamID);
+    addFocusWidget(myRamGrid);
+
+    t = new StaticTextWidget(boss, font, xpos, ypos + 2,
+                             lwidth-2, fontHeight,
+                             "00:", kTextAlignLeft);
+    for(int col = 0; col < 16; ++col)
+    {
+      t = new StaticTextWidget(boss, font, xpos + col*myRamGrid->colWidth() + lwidth + 7,
+                               ypos - lineHeight,
+                               fontWidth, fontHeight,
+                               Debugger::to_hex_4(col),
+                               kTextAlignLeft);
+    }
   
-  xpos = 20;  ypos += 2 * lineHeight;
-  t = new StaticTextWidget(boss, font, xpos, ypos, 6*fontWidth, fontHeight,
-                           "Label:", kTextAlignLeft);
-  xpos += 6*fontWidth + 5;
-  myLabel = new EditTextWidget(boss, font, xpos, ypos-2, 15*fontWidth,
-                               lineHeight, "");
-  myLabel->setEditable(false);
+    xpos = 20;  ypos += 2 * lineHeight;
+    t = new StaticTextWidget(boss, font, xpos, ypos, 6*fontWidth, fontHeight,
+                             "Label:", kTextAlignLeft);
+    xpos += 6*fontWidth + 5;
+    myLabel = new EditTextWidget(boss, font, xpos, ypos-2, 15*fontWidth,
+                                 lineHeight, "");
+    myLabel->setEditable(false);
 
-  xpos += 15*fontWidth + 20;
-  t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
-                           "Dec:", kTextAlignLeft);
-  xpos += 4*fontWidth + 5;
-  myDecValue = new EditTextWidget(boss, font, xpos, ypos-2, 4*fontWidth,
-                                  lineHeight, "");
-  myDecValue->setEditable(false);
+    xpos += 15*fontWidth + 20;
+    t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
+                             "Dec:", kTextAlignLeft);
+    xpos += 4*fontWidth + 5;
+    myDecValue = new EditTextWidget(boss, font, xpos, ypos-2, 4*fontWidth,
+                                    lineHeight, "");
+    myDecValue->setEditable(false);
 
-  xpos += 4*fontWidth + 20;
-  t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
-                           "Bin:", kTextAlignLeft);
-  xpos += 4*fontWidth + 5;
-  myBinValue = new EditTextWidget(boss, font, xpos, ypos-2, 9*fontWidth,
-                                  lineHeight, "");
-  myBinValue->setEditable(false);
+    xpos += 4*fontWidth + 20;
+    t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
+                             "Bin:", kTextAlignLeft);
+    xpos += 4*fontWidth + 5;
+    myBinValue = new EditTextWidget(boss, font, xpos, ypos-2, 9*fontWidth,
+                                    lineHeight, "");
+    myBinValue->setEditable(false);
+    ypos += lineHeight + 10;
+  }
 
   // Color registers
   const char* regNames[] = { "COLUP0:", "COLUP1:", "COLUPF:", "COLUBK:" };
-  xpos = 10;  ypos += 3*lineHeight;
+  xpos = 10;  ypos += lineHeight + 5;
   for(int row = 0; row < 4; ++row)
   {
     t = new StaticTextWidget(boss, font, xpos, ypos + row*lineHeight + 2,
@@ -272,22 +232,11 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
                        "CxClr", kCxclrCmd);
   b->setTarget(this);
 
-  // Set the strings to be used in the grPx registers
-  // We only do this once because it's the state that changes, not the strings
-  const char* offstr[] = { "0", "0", "0", "0", "0", "0", "0", "0" };
-  const char* onstr[]  = { "1", "1", "1", "1", "1", "1", "1", "1" };
-  StringList off, on;
-  for(int i = 0; i < 8; ++i)
-  {
-    off.push_back(offstr[i]);
-    on.push_back(onstr[i]);
-  }
-
   ////////////////////////////
   // P0 register info
   ////////////////////////////
   // grP0
-  xpos = 10;  ypos = 13*lineHeight;
+  xpos = 10;  ypos = buttonY + 2*lineHeight;
   t = new StaticTextWidget(boss, font, xpos, ypos+2,
                            7*fontWidth, fontHeight,
                            "P0: GR:", kTextAlignLeft);
@@ -648,7 +597,7 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   int addr, value;
   string buf;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
 
   switch(cmd)
@@ -800,7 +749,9 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           addr  = myRamGrid->getSelectedAddr();
           value = myRamGrid->getSelectedValue();
 
-          myLabel->setEditString(dbg.equates()->getLabel(addr));
+          // We're using the read-addresses here
+          // Should we also add write-addresses, or remove this entirely?
+          myLabel->setEditString(dbg.equates().getLabel(addr, true));
 
           myDecValue->setEditString(dbg.valueToString(value, kBASE_10));
           myBinValue->setEditString(dbg.valueToString(value, kBASE_2));
@@ -929,20 +880,23 @@ void TiaWidget::fillGrid()
   IntArray vlist;
   BoolArray blist, changed, grNew, grOld;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
-  TiaState state    = (TiaState&) tia.getState();
-  TiaState oldstate = (TiaState&) tia.getOldState();
+  TiaState& state    = (TiaState&) tia.getState();
+  TiaState& oldstate = (TiaState&) tia.getOldState();
 
   // TIA RAM
-  alist.clear();  vlist.clear();  changed.clear();
-  for(unsigned int i = 0; i < 16; i++)
+  if(myRamGrid)
   {
-    alist.push_back(i);
-    vlist.push_back(state.ram[i]);
-    changed.push_back(state.ram[i] != oldstate.ram[i]);
+    alist.clear();  vlist.clear();  changed.clear();
+    for(unsigned int i = 0; i < 16; i++)
+    {
+      alist.push_back(i);
+      vlist.push_back(state.ram[i]);
+      changed.push_back(state.ram[i] != oldstate.ram[i]);
+    }
+    myRamGrid->setList(alist, vlist, changed);
   }
-  myRamGrid->setList(alist, vlist, changed);
 
   // Color registers
   alist.clear();  vlist.clear();  changed.clear();
@@ -1108,22 +1062,22 @@ void TiaWidget::changeColorRegs()
   switch(addr)
   {
     case kCOLUP0Addr:
-      instance()->debugger().tiaDebug().coluP0(value);
+      instance().debugger().tiaDebug().coluP0(value);
       myCOLUP0Color->setColor(value);
       break;
 
     case kCOLUP1Addr:
-      instance()->debugger().tiaDebug().coluP1(value);
+      instance().debugger().tiaDebug().coluP1(value);
       myCOLUP1Color->setColor(value);
       break;
 
     case kCOLUPFAddr:
-      instance()->debugger().tiaDebug().coluPF(value);
+      instance().debugger().tiaDebug().coluPF(value);
       myCOLUPFColor->setColor(value);
       break;
 
     case kCOLUBKAddr:
-      instance()->debugger().tiaDebug().coluBK(value);
+      instance().debugger().tiaDebug().coluBK(value);
       myCOLUBKColor->setColor(value);
       break;
   }
