@@ -8,75 +8,65 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartCV.cxx,v 1.14 2007/01/14 16:17:53 stephena Exp $
+// $Id: CartCV.cxx 1862 2009-08-27 22:59:14Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
 #include "CartCV.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCV::CartridgeCV(const uInt8* image, uInt32 size)
+  : myROM(0),
+    mySize(size)
 {
-  uInt32 addr;
-  if(size == 2048)
-  {
-    // Copy the ROM image into my buffer
-    for(uInt32 addr = 0; addr < 2048; ++addr)
-    {
-      myImage[addr] = image[addr];
-    }
+  myROM = new uInt8[mySize];
+  memcpy(myROM, image, mySize);
 
-    // Initialize RAM with random values
-    class Random random;
-    for(uInt32 i = 0; i < 1024; ++i)
-    {
-      myRAM[i] = random.next();
-    }
-  }
-  else if(size == 4096)
-  {
-    // The game has something saved in the RAM
-    // Usefull for MagiCard program listings
+  reset();
 
-    // Copy the ROM image into my buffer
-    for(addr = 0; addr < 2048; ++addr)
-    {
-      myImage[addr] = image[addr + 2048];
-    }
-
-    // Copy the RAM image into my buffer
-    for(addr = 0; addr < 1024; ++addr)
-    {
-      myRAM[addr] = image[addr];
-    }
-
-  }
+  // This cart contains 1024 bytes extended RAM @ 0x1000
+  registerRamArea(0x1000, 1024, 0x00, 0x400);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCV::~CartridgeCV()
 {
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeCV::name() const
-{
-  return "CartridgeCV";
+  delete[] myROM;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCV::reset()
 {
+  if(mySize == 2048)
+  {
+    // Copy the ROM data into my buffer
+    memcpy(myImage, myROM, 2048);
+
+    // Initialize RAM with random values
+    class Random random;
+    for(uInt32 i = 0; i < 1024; ++i)
+      myRAM[i] = random.next();
+  }
+  else if(mySize == 4096)
+  {
+    // The game has something saved in the RAM
+    // Useful for MagiCard program listings
+
+    // Copy the ROM data into my buffer
+    memcpy(myImage, myROM + 2048, 2048);
+
+    // Copy the RAM image into my buffer
+    memcpy(myRAM, myROM, 1024);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -90,13 +80,13 @@ void CartridgeCV::install(System& system)
   assert((0x1800 & mask) == 0);
 
   System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
 
   // Map ROM image into the system
   for(uInt32 address = 0x1800; address < 0x2000; address += (1 << shift))
   {
+    access.device = this;
     access.directPeekBase = &myImage[address & 0x07FF];
+    access.directPokeBase = 0;
     mySystem->setPageAccess(address >> mySystem->pageShift(), access);
   }
 
@@ -122,7 +112,18 @@ void CartridgeCV::install(System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeCV::peek(uInt16 address)
 {
-  return myImage[address & 0x07FF];
+  // Reading from the write port triggers an unwanted write
+  // The value written to RAM is somewhat undefined, so we use 0
+  // Thanks to Kroko of AtariAge for this advice and code idea
+  if((address & 0x0FFF) < 0x0800)  // Write port is at 0xF400 - 0xF800 (1024 bytes)
+  {                                // Read port is handled in ::install()
+    if(myBankLocked) return 0;
+    else return myRAM[address & 0x03FF] = 0;
+  }  
+  else
+  {
+    return myImage[address & 0x07FF];
+  }  
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -131,63 +132,6 @@ void CartridgeCV::poke(uInt16, uInt8)
   // This is ROM so poking has no effect :-)
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeCV::save(Serializer& out)
-{
-  string cart = name();
-
-  try
-  {
-    out.putString(cart);
-
-    // Output RAM
-    out.putInt(1024);
-    for(uInt32 addr = 0; addr < 1024; ++addr)
-      out.putInt(myRAM[addr]);
-  }
-  catch(const char* msg)
-  {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
-    return false;
-  }
-
-  return true;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeCV::load(Deserializer& in)
-{
-  string cart = name();
-
-  try
-  {
-    if(in.getString() != cart)
-      return false;
-
-    // Input RAM
-    uInt32 limit = (uInt32) in.getInt();
-    for(uInt32 addr = 0; addr < limit; ++addr)
-      myRAM[addr] = (uInt8) in.getInt();
-  }
-  catch(const char* msg)
-  {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
-    return false;
-  }
-
-  return true;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCV::bank(uInt16 bank)
 {
@@ -220,3 +164,50 @@ uInt8* CartridgeCV::getImage(int& size)
   size = 2048;
   return &myImage[0];
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeCV::save(Serializer& out) const
+{
+  const string& cart = name();
+
+  try
+  {
+    out.putString(cart);
+
+    // Output RAM
+    out.putInt(1024);
+    for(uInt32 addr = 0; addr < 1024; ++addr)
+      out.putByte((char)myRAM[addr]);
+  }
+  catch(const char* msg)
+  {
+    cerr << "ERROR: CartridgeCV::save" << endl << "  " << msg << endl;
+    return false;
+  }
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeCV::load(Serializer& in)
+{
+  const string& cart = name();
+
+  try
+  {
+    if(in.getString() != cart)
+      return false;
+
+    // Input RAM
+    uInt32 limit = (uInt32) in.getInt();
+    for(uInt32 addr = 0; addr < limit; ++addr)
+      myRAM[addr] = (uInt8) in.getByte();
+  }
+  catch(const char* msg)
+  {
+    cerr << "ERROR: CartridgeCV::load" << endl << "  " << msg << endl;
+    return false;
+  }
+
+  return true;
+}
