@@ -8,73 +8,47 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartMC.cxx,v 1.12 2007/01/14 16:17:55 stephena Exp $
+// $Id: CartMC.cxx 1862 2009-08-27 22:59:14Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
 #include "CartMC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeMC::CartridgeMC(const uInt8* image, uInt32 size)
   : mySlot3Locked(false)
 {
-  uInt32 i;
-
   // Make sure size is reasonable
-  assert(size <= 128 * 1024);
-
-  // Allocate array for the cart's RAM
-  myRAM = new uInt8[32 * 1024];
-
-  // Initialize RAM with random values
-  class Random random;
-  for(i = 0; i < 32 * 1024; ++i)
-  {
-    myRAM[i] = random.next();
-  }
-
-  // Allocate array for the ROM image
-  myImage = new uInt8[128 * 1024];
+  assert(size <= 131072);
 
   // Set the contents of the entire ROM to 0
-  for(i = 0; i < 128 * 1024; ++i)
-  {
-    myImage[i] = 0;
-  }
+  memset(myImage, 0, 131072);
 
   // Copy the ROM image to the end of the ROM buffer
-  for(i = 0; i < size; ++i)
-  {
-    myImage[128 * 1024 - size + i] = image[i];
-  }
+  memcpy(myImage + 131072 - size, image, size);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeMC::~CartridgeMC()
 {
-  delete[] myRAM;
-  delete[] myImage;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeMC::name() const
-{
-  return "CartridgeMC";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMC::reset()
 {
+  // Initialize RAM with random values
+  class Random random;
+  for(uInt32 i = 0; i < 32768; ++i)
+    myRAM[i] = random.next();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -117,7 +91,7 @@ void CartridgeMC::install(System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeMC::peek(uInt16 address)
 {
-  address = address & 0x1FFF;
+  address &= 0x1FFF;
 
   // Accessing the RESET vector so lets handle the powerup special case
   if((address == 0x1FFC) || (address == 0x1FFD))
@@ -154,7 +128,7 @@ uInt8 CartridgeMC::peek(uInt16 address)
     if(block & 0x80)
     {
       // ROM access
-      return myImage[(uInt32)(block & 0x7F) * 1024 + (address & 0x03FF)];
+      return myImage[(uInt32)((block & 0x7F) << 10) + (address & 0x03FF)];
     }
     else
     {
@@ -162,12 +136,12 @@ uInt8 CartridgeMC::peek(uInt16 address)
       if(address & 0x0200)
       {
         // Reading from the read port of the RAM block
-        return myRAM[(uInt32)(block & 0x3F) * 512 + (address & 0x01FF)];
+        return myRAM[(uInt32)((block & 0x3F) << 9) + (address & 0x01FF)];
       }
       else
       {
         // Oops, reading from the write port of the RAM block!
-        myRAM[(uInt32)(block & 0x3F) * 512 + (address & 0x01FF)] = 0;
+        myRAM[(uInt32)((block & 0x3F) << 9) + (address & 0x01FF)] = 0;
         return 0;
       }
     }
@@ -177,7 +151,7 @@ uInt8 CartridgeMC::peek(uInt16 address)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMC::poke(uInt16 address, uInt8 value)
 {
-  address = address & 0x1FFF;
+  address &= 0x1FFF;
 
   // Accessing the RESET vector so lets handle the powerup special case
   if((address == 0x1FFC) || (address == 0x1FFD))
@@ -214,16 +188,50 @@ void CartridgeMC::poke(uInt16 address, uInt8 value)
     if(!(block & 0x80) && !(address & 0x0200))
     {
       // Handle the write to RAM
-      myRAM[(uInt32)(block & 0x3F) * 512 + (address & 0x01FF)] = value;
+      myRAM[(uInt32)((block & 0x3F) << 9) + (address & 0x01FF)] = value;
     }
   }  
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeMC::save(Serializer& out)
+void CartridgeMC::bank(uInt16 b)
+{
+  // TODO - add support for debugger
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeMC::bank()
+{
+  // TODO - add support for debugger
+  return 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeMC::bankCount()
+{
+  // TODO - add support for debugger
+  return 1;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMC::patch(uInt16 address, uInt8 value)
+{
+  // TODO - add support for debugger
+  return false;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeMC::getImage(int& size)
+{
+  size = 128 * 1024; // FIXME: keep track of original size
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMC::save(Serializer& out) const
 {
   uInt32 i;
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -232,21 +240,16 @@ bool CartridgeMC::save(Serializer& out)
     // The currentBlock array
     out.putInt(4);
     for(i = 0; i < 4; ++i)
-      out.putInt(myCurrentBlock[i]);
+      out.putByte((char)myCurrentBlock[i]);
 
     // The 32K of RAM
     out.putInt(32 * 1024);
     for(i = 0; i < 32 * 1024; ++i)
-      out.putInt(myRAM[i]);
+      out.putByte((char)myRAM[i]);
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeMC::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -254,10 +257,10 @@ bool CartridgeMC::save(Serializer& out)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeMC::load(Deserializer& in)
+bool CartridgeMC::load(Serializer& in)
 {
   uInt32 i;
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -269,57 +272,18 @@ bool CartridgeMC::load(Deserializer& in)
     // The currentBlock array
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myCurrentBlock[i] = (uInt8) in.getInt();
+      myCurrentBlock[i] = (uInt8) in.getByte();
 
     // The 32K of RAM
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getInt();
+      myRAM[i] = (uInt8) in.getByte();
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeMC::load" << endl << "  " << msg << endl;
     return false;
   }
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeMC::bank(uInt16 b)
-{
-  // TODO: add support for debugger
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeMC::bank()
-{
-  // TODO: add support for debugger
-  return 0;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeMC::bankCount()
-{
-  // TODO: add support for debugger
-  return 1;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeMC::patch(uInt16 address, uInt8 value)
-{
-  // TODO: implement
-  return false;
-} 
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeMC::getImage(int& size)
-{
-  size = 128 * 1024; // FIXME: keep track of original size
-  return &myImage[0];
-}
