@@ -8,21 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Settings.cxx,v 1.115 2007/01/03 17:37:46 stephena Exp $
+// $Id: Settings.cxx 1852 2009-08-07 11:45:14Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <sstream>
 #include <fstream>
+#include <algorithm>
+
+#include "bspf.hxx"
 
 #include "OSystem.hxx"
 #include "Version.hxx"
-#include "bspf.hxx"
+
 #include "Settings.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -34,22 +37,35 @@ Settings::Settings(OSystem* osystem)
 
   // Add options that are common to all versions of Stella
   setInternal("video", "soft");
-  setInternal("dirtyrects", "false"); // This only becomes useful at high resolutions
 
+  // OpenGL specific options
   setInternal("gl_filter", "nearest");
-  setInternal("gl_aspect", "2.0");
-  setInternal("gl_fsmax", "true");
+  setInternal("gl_aspectn", "100");
+  setInternal("gl_aspectp", "100");
+  setInternal("gl_fsmax", "false");
   setInternal("gl_lib", "libGL.so");
   setInternal("gl_vsync", "false");
+  setInternal("gl_texrect", "false");
+//  setInternal("gl_accel", "true");
 
-  setInternal("scale_ui", "zoom2x");
-  setInternal("scale_tia", "zoom2x");
-  setInternal("fullscreen", "false");
+  // Framebuffer-related options
+  setInternal("tia_filter", "zoom2x");
+  setInternal("fullscreen", "0");
+  setInternal("fullres", "auto");
   setInternal("center", "true");
   setInternal("grabmouse", "false");
   setInternal("palette", "standard");
   setInternal("colorloss", "false");
+  setInternal("timing", "sleep");
 
+  // TV filter options
+  setInternal("tv_tex", "off");
+  setInternal("tv_bleed", "off");
+  setInternal("tv_noise", "off");
+//  setInternal("tv_curve", "false");  // not yet implemented
+  setInternal("tv_phos", "false");
+
+  // Sound options
   setInternal("sound", "true");
   setInternal("fragsize", "512");
   setInternal("freq", "31400");
@@ -57,32 +73,53 @@ Settings::Settings(OSystem* osystem)
   setInternal("volume", "100");
   setInternal("clipvol", "true");
 
+  // Input event options
   setInternal("keymap", "");
   setInternal("joymap", "");
   setInternal("joyaxismap", "");
   setInternal("joyhatmap", "");
-  setInternal("paddle", "0");
+  setInternal("joydeadzone", "0");
+  setInternal("joyallow4", "false");
+  setInternal("pspeed", "6");
   setInternal("sa1", "left");
   setInternal("sa2", "right");
-  setInternal("p0speed", "50");
-  setInternal("p1speed", "50");
-  setInternal("p2speed", "50");
-  setInternal("p3speed", "50");
-  setInternal("pthresh", "600");
 
-  setInternal("showinfo", "false");
-
-  setInternal("ssdir", string(".") + BSPF_PATH_SEPARATOR);
+  // Snapshot options
+  setInternal("ssdir", "");
   setInternal("sssingle", "false");
+  setInternal("ss1x", "false");
 
-  setInternal("romdir", "");
-  setInternal("rombrowse", "true");
+  // Config files and paths
+  setInternal("romdir", "~");
+  setInternal("statedir", "");
+  setInternal("cheatfile", "");
+  setInternal("palettefile", "");
+  setInternal("propsfile", "");
+  setInternal("eepromdir", "");
+
+  // ROM browser options
+  setInternal("uselauncher", "true");
+  setInternal("launcherres", "640x480");
+  setInternal("launcherfont", "medium");
+  setInternal("launcherexts", "allfiles");
+  setInternal("romviewer", "0");
   setInternal("lastrom", "");
-  setInternal("modtime", "");
-  setInternal("debugheight", "0");
-  setInternal("launchersize", "2");
+
+  // UI-related options
+  setInternal("debuggerres", "1030x690");
   setInternal("uipalette", "0");
+  setInternal("listdelay", "300");
+  setInternal("mwheel", "4");
+
+  // Misc options
   setInternal("autoslot", "false");
+  setInternal("showinfo", "false");
+  setInternal("tiafloat", "true");
+  setInternal("avoxport", "");
+  setInternal("stats", "false");
+  setInternal("audiofirst", "true");
+  setInternal("fastscbios", "false");
+  setExternal("romloadcount", "0");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -101,7 +138,7 @@ void Settings::loadConfig()
   ifstream in(myOSystem->configFile().c_str());
   if(!in || !in.is_open())
   {
-    cout << "Error: Couldn't load settings file\n";
+    cout << "ERROR: Couldn't load settings file\n";
     return;
   }
 
@@ -138,71 +175,51 @@ void Settings::loadConfig()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Settings::loadCommandLine(int argc, char** argv)
+string Settings::loadCommandLine(int argc, char** argv)
 {
   for(int i = 1; i < argc; ++i)
   {
     // strip off the '-' character
     string key = argv[i];
-    if(key[0] != '-')
-      return true;     // stop processing here, ignore the remaining items
+    if(key[0] == '-')
+    {
+      key = key.substr(1, key.length());
 
-    key = key.substr(1, key.length());
+      // Take care of the arguments which are meant to be executed immediately
+      // (and then Stella should exit)
+      if(key == "help" || key == "listrominfo")
+      {
+        setExternal(key, "true");
+        return "";
+      }
 
-    // Take care of the arguments which are meant to be executed immediately
-    // (and then Stella should exit)
-    if(key == "help")
-    {
-      usage();
-      return false;
-    }
-    else if(key == "listrominfo")
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "rominfo")
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "debug") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "holdreset") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "holdselect") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "holdbutton0") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
+      // Take care of arguments without an option
+      if(key == "rominfo" || key == "debug" || key == "holdreset" ||
+         key == "holdselect" || key == "holdbutton0" || key == "takesnapshot")
+      {
+        setExternal(key, "true");
+        continue;
+      }
 
-    if(++i >= argc)
-    {
-      cerr << "Missing argument for '" << key << "'" << endl;
-      return false;
-    }
-    string value = argv[i];
+      if(++i >= argc)
+      {
+        cerr << "Missing argument for '" << key << "'" << endl;
+        return "";
+      }
+      string value = argv[i];
 
-    // Settings read from the commandline must not be saved to 
-    // the rc-file, unless they were previously set
-    if(int idx = getInternalPos(key) != -1)
-      setInternal(key, value, idx);   // don't set initialValue here
+      // Settings read from the commandline must not be saved to 
+      // the rc-file, unless they were previously set
+      if(int idx = getInternalPos(key) != -1)
+        setInternal(key, value, idx);   // don't set initialValue here
+      else
+        setExternal(key, value);
+    }
     else
-      setExternal(key, value);
+      return key;
   }
 
-  return true;
+  return "";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -212,54 +229,61 @@ void Settings::validate()
   int i;
 
   s = getString("video");
-  if(s != "soft" && s != "gl")
-    setInternal("video", "soft");
+  if(s != "soft" && s != "gl")  setInternal("video", "soft");
+
+  s = getString("timing");
+  if(s != "sleep" && s != "busy")  setInternal("timing", "sleep");
 
 #ifdef DISPLAY_OPENGL
   s = getString("gl_filter");
-  if(s != "linear" && s != "nearest")
-    setInternal("gl_filter", "nearest");
+  if(s != "linear" && s != "nearest")  setInternal("gl_filter", "nearest");
 
-  float f = getFloat("gl_aspect");
-  if(f < 1.1 || f > 2.0)
-    setInternal("gl_aspect", "2.0");
+  i = getInt("gl_aspectn");
+  if(i < 80 || i > 120)  setInternal("gl_aspectn", "100");
+  i = getInt("gl_aspectp");
+  if(i < 80 || i > 120)  setInternal("gl_aspectp", "100");
 #endif
 
 #ifdef SOUND_SUPPORT
   i = getInt("volume");
-  if(i < 0 || i > 100)
-    setInternal("volume", "100");
+  if(i < 0 || i > 100)    setInternal("volume", "100");
   i = getInt("freq");
-  if(i < 0 || i > 48000)
-    setInternal("freq", "31400");
+  if(i < 0 || i > 48000)  setInternal("freq", "31400");
   i = getInt("tiafreq");
-  if(i < 0 || i > 48000)
-    setInternal("tiafreq", "31400");
+  if(i < 0 || i > 48000)  setInternal("tiafreq", "31400");
 #endif
 
-  s = getString("scale_ui");
-  if(s != "zoom1x" && s != "zoom2x" && s != "zoom3x" &&
-     s != "zoom4x" && s != "zoom5x" && s != "zoom6x")
-    setInternal("scale_ui", "zoom1x");
+  i = getInt("joydeadzone");
+  if(i < 0)        setInternal("joydeadzone", "0");
+  else if(i > 29)  setInternal("joydeadzone", "29");
 
-  s = getString("scale_tia");
-  if(s != "zoom1x" && s != "zoom2x" && s != "zoom3x" &&
-     s != "zoom4x" && s != "zoom5x" && s != "zoom6x")
-    setInternal("scale_tia", "zoom1x");
-
-  i = getInt("paddle");
-  if(i < 0 || i > 3)
-    setInternal("paddle", "0");
-
-  i = getInt("pthresh");
-  if(i < 400)
-    setInternal("pthresh", "400");
-  else if(i > 800)
-    setInternal("pthresh", "800");
+  i = getInt("pspeed");
+  if(i < 1)        setInternal("pspeed", "1");
+  else if(i > 15)  setInternal("pspeed", "15");
 
   s = getString("palette");
-  if(s != "standard" && s != "original" && s != "z26" && s != "user")
+  if(s != "standard" && s != "z26" && s != "user")
     setInternal("palette", "standard");
+
+  s = getString("launcherfont");
+  if(s != "small" && s != "medium" && s != "large")
+    setInternal("launcherfont", "medium");
+
+  i = getInt("romviewer");
+  if(i < 0)       setInternal("romviewer", "0");
+  else if(i > 2)  setInternal("romviewer", "2");
+
+  s = getString("tv_tex");
+  if(s != "normal" && s != "stag")
+    setInternal("tv_tex", "off");
+
+  s = getString("tv_bleed");
+  if(s != "low" && s != "medium" && s != "high")
+    setInternal("tv_bleed", "off");
+
+  s = getString("tv_noise");
+  if(s != "low" && s != "medium" && s != "high")
+    setInternal("tv_noise", "off");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -271,6 +295,7 @@ void Settings::usage()
     << endl
     << "Usage: stella [options ...] romfile" << endl
     << "       Run without any options or romfile to use the ROM launcher" << endl
+    << "       Consult the manual for more in-depth information" << endl
     << endl
     << "Valid options are:" << endl
     << endl
@@ -283,22 +308,37 @@ void Settings::usage()
     << "  -gl_filter    <type>         Type is one of the following:\n"
     << "                 nearest         Normal scaling (GL_NEAREST)\n"
     << "                 linear          Blurred scaling (GL_LINEAR)\n"
-    << "  -gl_aspect    <number>       Scale the width by the given amount\n"
-    << "  -gl_fsmax     <1|0>          Use the largest available screenmode in fullscreen OpenGL\n"
+    << "  -gl_aspectn   <number>       Scale the TIA width by the given percentage in NTSC mode\n"
+    << "  -gl_aspectp   <number>       Scale the TIA width by the given percentage in PAL mode\n"
+    << "  -gl_fsmax     <1|0>          Stretch GL image in fullscreen emulation mode\n"
     << "  -gl_vsync     <1|0>          Enable synchronize to vertical blank interrupt\n"
+    << "  -gl_texrect   <1|0>          Enable GL_TEXTURE_RECTANGLE extension\n"
+//    << "  -gl_accel     <1|0>          Enable SDL_GL_ACCELERATED_VISUAL\n"
+    << "  -tv_tex       <off|type>     OpenGL TV texturing, type is one of the following:\n"
+    << "                 normal         Aligned in a grid\n"
+    << "                 stag           Aligned in a staggered grid\n"
+    << "  -tv_bleed     <off|type>     OpenGL TV color bleed, type is one of the following:\n"
+    << "                 low            \n"
+    << "                 medium         \n"
+    << "                 high           \n"
+    << "  -tv_noise     <off|type>     OpenGL TV RF noise emulation, type is one of the following:\n"
+    << "                 low            \n"
+    << "                 medium         \n"
+    << "                 high           \n"
+    << "  -tv_phos      <1|0>          OpenGL TV phosphor burn-off\n"
     << endl
   #endif
-    << "  -scale_tia    <scaler>       Use the specified scaler in emulation mode\n"
-    << "  -scale_ui     <scaler>       Use the specified scaler in non-emulation mode (ROM browser/debugger)\n"
-    << "  -fullscreen   <1|0>          Play the game in fullscreen mode\n"
+    << "  -tia_filter   <filter>       Use the specified filter in emulation mode\n"
+    << "  -fullscreen   <1|0|-1>       Use fullscreen mode (1 or 0), or disable switching to fullscreen entirely\n"
+    << "  -fullres      <auto|WxH>     The resolution to use in fullscreen mode\n"
     << "  -center       <1|0>          Centers game window (if possible)\n"
     << "  -grabmouse    <1|0>          Keeps the mouse in the game window\n"
-    << "  -palette      <original|     Use the specified color palette\n"
-    << "                 standard|\n"
+    << "  -palette      <standard|     Use the specified color palette\n"
     << "                 z26|\n"
     << "                 user>\n"
     << "  -colorloss    <1|0>          Enable PAL color-loss effect\n"
-    << "  -framerate    <number>       Display the given number of frames per second\n"
+    << "  -framerate    <number>       Display the given number of frames per second (0 to auto-calculate)\n"
+    << "  -timing       <sleep|busy>   Use the given type of wait between frames\n"
     << endl
   #ifdef SOUND_SUPPORT
     << "  -sound        <1|0>          Enable sound generation\n"
@@ -310,42 +350,56 @@ void Settings::usage()
     << endl
   #endif
     << "  -cheat        <code>         Use the specified cheatcode (see manual for description)\n"
-    << "  -showinfo     <1|0>          Shows some game info\n"
-    << "  -paddle       <0|1|2|3>      Indicates which paddle the mouse should emulate\n"
+    << "  -showinfo     <1|0>          Shows some game info on commandline\n"
+    << "  -joydeadzone  <number>       Sets 'deadzone' area for analog joysticks (0-29)\n"
+    << "  -joyallow4    <1|0>          Allow all 4 directions on a joystick to be pressed simultaneously\n"
+    << "  -pspeed       <number>       Speed of digital emulated paddle movement (1-15)\n"
     << "  -sa1          <left|right>   Stelladaptor 1 emulates specified joystick port\n"
     << "  -sa2          <left|right>   Stelladaptor 2 emulates specified joystick port\n"
-    << "  -p0speed      <number>       Speed of emulated mouse movement for paddle 0 (0-100)\n"
-    << "  -p1speed      <number>       Speed of emulated mouse movement for paddle 1 (0-100)\n"
-    << "  -p2speed      <number>       Speed of emulated mouse movement for paddle 2 (0-100)\n"
-    << "  -p3speed      <number>       Speed of emulated mouse movement for paddle 3 (0-100)\n"
-    << "  -pthresh      <number>       Set threshold for eliminating paddle jitter\n"
-    << "  -rombrowse    <1|0>          Use ROM browser mode (shows files and folders)\n"
     << "  -autoslot     <1|0>          Automatically switch to next save slot when state saving\n"
-  #ifdef UNIX
-    << "  -accurate     <1|0>          Accurate game timing (uses more CPU)\n"
-  #endif
+    << "  -audiofirst   <1|0>          Initial audio before video (required for some ATI video cards)\n"
+    << "  -fastscbios   <1|0>          Disable Supercharger BIOS progress loading bars\n"
     << "  -ssdir        <path>         The directory to save snapshot files to\n"
     << "  -sssingle     <1|0>          Generate single snapshot instead of many\n"
+    << "  -ss1x         <1|0>          Generate TIA snapshot in 1x mode (ignore scaling/effects)\n"
     << endl
-    << "  -listrominfo                 Display contents of stella.pro, one line per ROM entry\n"
     << "  -rominfo      <rom>          Display detailed information for the given ROM\n"
-    << "  -launchersize <1|2|3>        Set the size of the ROM launcher\n"
+    << "  -listrominfo                 Display contents of stella.pro, one line per ROM entry\n"
+    << "  -uselauncher  <1|0>          Use the built-in ROM launcher\n"
+    << "  -launcherres  <WxH>          The resolution to use in ROM launcher mode\n"
+    << "  -launcherfont <small|medium| Use the specified font in the ROM launcher\n"
+    << "                 large>\n"
+    << "  -launcherexts <allfiles|     Show files with the given extensions in ROM launcher\n"
+    << "                 allroms|        (exts is a ':' separated list of extensions)\n"
+    << "                 exts\n"
+    << "  -romviewer    <0|1|2>        Show ROM info viewer at given zoom level in ROM launcher (0 for off)\n"
     << "  -uipalette    <1|2>          Used the specified palette for UI elements\n"
+    << "  -listdelay    <delay>        Time to wait between keypresses in list widgets (300-1000)\n"
+    << "  -mwheel       <lines>        Number of lines the mouse wheel will scroll in UI\n"
+    << "  -statedir     <dir>          Directory in which to save state files\n"
+    << "  -cheatfile    <file>         Full pathname of cheatfile database\n"
+    << "  -palettefile  <file>         Full pathname of user-defined palette file\n"
+    << "  -propsfile    <file>         Full pathname of ROM properties file\n"
+    << "  -eepromdir    <dir>          Directory in which to save EEPROM files\n"
+    << "  -avoxport     <name>         The name of the serial port where an AtariVox is connected\n"
     << "  -help                        Show the text you're now reading\n"
   #ifdef DEBUGGER_SUPPORT
     << endl
     << " The following options are meant for developers\n"
     << " Arguments are more fully explained in the manual\n"
     << endl
+    << "   -debuggerres  <WxH>         The resolution to use in debugger mode\n"
     << "   -break        <address>     Set a breakpoint at 'address'\n"
-    << "   -debugheight  <number>      Set height of debugger in lines of text (NOT pixels)\n"
     << "   -debug                      Start in debugger mode\n"
     << "   -holdreset                  Start the emulator with the Game Reset switch held down\n"
     << "   -holdselect                 Start the emulator with the Game Select switch held down\n"
     << "   -holdbutton0                Start the emulator with the left joystick button held down\n"
+    << "   -stats        <1|0>         Overlay console info during emulation\n"
+    << "   -tiafloat     <1|0>         Set unused TIA pins floating on a read/peek\n"
     << endl
-    << "   -pro         <props file>   Use the given properties file instead of stella.pro\n"
-    << "   -type        <arg>          Sets the 'Cartridge.Type' property\n"
+    << "   -bs          <arg>          Sets the 'Cartridge.Type' (bankswitch) property\n"
+    << "   -type        <arg>          Same as using -bs\n"
+    << "   -channels    <arg>          Sets the 'Cartridge.Sound' property\n"
     << "   -ld          <arg>          Sets the 'Console.LeftDifficulty' property\n"
     << "   -rd          <arg>          Sets the 'Console.RightDifficulty' property\n"
     << "   -tv          <arg>          Sets the 'Console.TelevisionType' property\n"
@@ -355,13 +409,10 @@ void Settings::usage()
     << "   -bc          <arg>          Same as using both -lc and -rc\n"
     << "   -cp          <arg>          Sets the 'Controller.SwapPaddles' property\n"
     << "   -format      <arg>          Sets the 'Display.Format' property\n"
-    << "   -xstart      <arg>          Sets the 'Display.XStart' property\n"
     << "   -ystart      <arg>          Sets the 'Display.YStart' property\n"
-    << "   -width       <arg>          Sets the 'Display.Width' property\n"
     << "   -height      <arg>          Sets the 'Display.Height' property\n"
     << "   -pp          <arg>          Sets the 'Display.Phosphor' property\n"
     << "   -ppblend     <arg>          Sets the 'Display.PPBlend' property\n"
-    << "   -hmove       <arg>          Sets the 'Emulation.HmoveBlanks' property\n"
   #endif
     << endl;
 #endif
@@ -388,7 +439,7 @@ void Settings::saveConfig()
   ofstream out(myOSystem->configFile().c_str());
   if(!out || !out.is_open())
   {
-    cout << "Error: Couldn't save settings file\n";
+    cout << "ERROR: Couldn't save settings file\n";
     return;
   }
 
@@ -462,6 +513,17 @@ void Settings::setString(const string& key, const string& value)
     setExternal(key, value);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Settings::getSize(const string& key, int& x, int& y) const
+{
+  char c;
+  string size = getString(key);
+  istringstream buf(size);
+  buf >> x >> c >> y;
+  if(c != 'x')
+    x = y = -1;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Settings::getInt(const string& key) const
 {
@@ -530,6 +592,14 @@ const string& Settings::getString(const string& key) const
     return EmptyString;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Settings::setSize(const string& key, const int value1, const int value2)
+{
+  ostringstream buf;
+  buf << value1 << "x" << value2;
+  setString(key, buf.str());
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Settings::getInternalPos(const string& key) const
 {
