@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Switches.cxx,v 1.7 2007/01/01 18:04:50 stephena Exp $
+// $Id: Switches.cxx 1862 2009-08-27 22:59:14Z stephena $
 //============================================================================
 
 #include "Event.hxx"
@@ -22,8 +22,8 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 Switches::Switches(const Event& event, const Properties& properties)
-    : myEvent(event),
-      mySwitches(0xFF)
+  : myEvent(event),
+    mySwitches(0xFF)
 {
   if(properties.get(Console_RightDifficulty) == "B")
   {
@@ -59,7 +59,7 @@ Switches::~Switches()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-uInt8 Switches::read()
+void Switches::update()
 {
   if(myEvent.get(Event::ConsoleColor) != 0)
   {
@@ -105,7 +105,34 @@ uInt8 Switches::read()
   {
     mySwitches |= 0x01;
   }
-
-  return mySwitches;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
+bool Switches::save(Serializer& out) const
+{
+  try
+  {
+    out.putByte((char)mySwitches);
+  }
+  catch(...)  // FIXME
+  {
+    cerr << "ERROR: Switches::save() exception\n";
+    return false;
+  }
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
+bool Switches::load(Serializer& in)
+{
+  try
+  {
+    mySwitches = (uInt8) in.getByte();
+  }
+  catch(...)  // FIXME
+  {
+    cerr << "ERROR: Switches::load() exception\n";
+    return false;
+  }
+  return true;
+}
