@@ -8,17 +8,16 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: NullDev.cxx,v 1.5 2007/01/01 18:04:51 stephena Exp $
+// $Id: NullDev.cxx 1849 2009-08-05 16:05:34Z stephena $
 //============================================================================
 
-#include "NullDev.hxx"
 #include "Serializer.hxx"
-#include "Deserializer.hxx"
+#include "NullDev.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 NullDevice::NullDevice()
@@ -30,12 +29,6 @@ NullDevice::~NullDevice()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* NullDevice::name() const
-{
-  return "NULL";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void NullDevice::reset()
 {
@@ -61,13 +54,13 @@ void NullDevice::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool NullDevice::save(Serializer& out)
+bool NullDevice::save(Serializer& out) const
 {
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool NullDevice::load(Deserializer& in)
+bool NullDevice::load(Serializer& in)
 {
   return true;
 }
