@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsGP2X.cxx,v 1.26 2007/01/01 19:55:55 stephena Exp $
+// $Id: SettingsGP2X.cxx 1724 2009-05-13 13:55:40Z stephena $
 // Modified on 2006/02/05 by Alex Zaballa for use on GP2X
 //============================================================================
 
@@ -44,6 +44,8 @@ SettingsGP2X::SettingsGP2X(OSystem* osystem)
   setInternal("p3speed", "15");
   setInternal("launchersize", "1");
   setInternal("uipalette", "2");
+  setInternal("tv_scale_width", "1.125");
+  setInternal("tv_scale_height", "1.2");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
