@@ -8,58 +8,75 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AboutDialog.cxx,v 1.18 2007/01/01 18:04:51 stephena Exp $
+// $Id: AboutDialog.cxx 1750 2009-06-04 13:52:10Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "Widget.hxx"
 #include "Dialog.hxx"
-#include "GuiUtils.hxx"
+#include "OSystem.hxx"
 #include "Version.hxx"
-#include "AboutDialog.hxx"
+#include "Widget.hxx"
 
-#define ADD_ATEXT(d) do { dsc[i] = d; i++; } while(0)
-#define ADD_ALINE ADD_ATEXT("")
+#include "AboutDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AboutDialog::AboutDialog(OSystem* osystem, DialogContainer* parent,
-                         const GUI::Font& font, int x, int y, int w, int h)
-    : Dialog(osystem, parent, x, y, w, h),
-      myPage(1),
-      myNumPages(6)
+                         const GUI::Font& font)
+  : Dialog(osystem, parent, 0, 0, 0, 0),
+    myPage(1),
+    myNumPages(5)
 {
+  const int lineHeight   = font.getLineHeight(),
+            fontWidth    = font.getMaxCharWidth(),
+            fontHeight   = font.getFontHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 20,
+            buttonHeight = font.getLineHeight() + 4;
+  int xpos, ypos;
   WidgetArray wid;
 
+  // Set real dimensions
+  _w = 52 * fontWidth + 8;
+  _h = 10 * lineHeight + 20;
+
   // Add Previous, Next and Close buttons
-  myPrevButton = addButton(font, 10, h - 24, "Previous", kPrevCmd);
+  xpos = 10;  ypos = _h - buttonHeight - 10;
+  myPrevButton =
+    new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
+                     "Previous", kPrevCmd);
   myPrevButton->clearFlags(WIDGET_ENABLED);
   wid.push_back(myPrevButton);
 
-  myNextButton = addButton(font, (kButtonWidth + 15), h - 24,
-                           "Next", kNextCmd);
+  xpos += buttonWidth + 7;
+  myNextButton =
+    new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
+                     "Next", kNextCmd);
   wid.push_back(myNextButton);
 
-  ButtonWidget* b = addButton(font, w - (kButtonWidth + 10), h - 24,
-                              "Close", kCloseCmd);
+  xpos = _w - buttonWidth - 10;
+  ButtonWidget* b =
+    new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
+                     "Close", kCloseCmd);
   wid.push_back(b);
   addOKWidget(b);  addCancelWidget(b);
 
-  myTitle = new StaticTextWidget(this, font, 5, 5, w - 10, font.getFontHeight(),
+  xpos = 5;  ypos = 5;
+  myTitle = new StaticTextWidget(this, font, xpos, ypos, _w - 10, fontHeight,
                                  "", kTextAlignCenter);
-  myTitle->setColor(kTextColorHi);
+  myTitle->setTextColor(kTextColorEm);
 
-  for(int i = 0; i < LINES_PER_PAGE; i++)
+  xpos = 10;  ypos += lineHeight + 4;
+  for(int i = 0; i < kLINES_PER_PAGE; i++)
   {
-    myDesc[i] = new StaticTextWidget(this, font, 10, 18 + (10 * i), w - 20,
-                                     font.getFontHeight(), "", kTextAlignLeft);
+    myDesc[i] = new StaticTextWidget(this, font, xpos, ypos, _w - 20,
+                                     fontHeight, "", kTextAlignLeft);
+    ypos += fontHeight;
   }
 
   addToFocusList(wid);
@@ -80,9 +97,10 @@ AboutDialog::~AboutDialog()
 //                  4 background (black)
 //                  5 emphasized text (red)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void AboutDialog::updateStrings(int page, int lines, string& title, string* &dsc)
+void AboutDialog::updateStrings(int page, int lines, string& title)
 {
-  dsc = new string[lines];
+#define ADD_ATEXT(d) do { myDescStr[i] = d; i++; } while(0)
+#define ADD_ALINE ADD_ATEXT("")
 
   int i = 0;
   switch(page)
@@ -90,9 +108,9 @@ void AboutDialog::updateStrings(int page, int lines, string& title, string* &dsc
     case 1:
       title = string("Stella ") + STELLA_VERSION;
       ADD_ATEXT("\\CA multi-platform Atari 2600 VCS emulator");
-      ADD_ATEXT(string("\\C\\c2") + instance()->features());
+      ADD_ATEXT(string("\\C\\c2") + instance().features());
       ADD_ALINE;
-      ADD_ATEXT("\\CCopyright (C) 1995-2007 The Stella team");
+      ADD_ATEXT("\\CCopyright (C) 1995-2009 The Stella team");
       ADD_ATEXT("\\Chttp://stella.sourceforge.net");
       ADD_ALINE;
       ADD_ATEXT("Stella is free software released under the GNU GPL");
@@ -104,57 +122,33 @@ void AboutDialog::updateStrings(int page, int lines, string& title, string* &dsc
       ADD_ATEXT("\\L\\c0""  Bradford W. Mott");
       ADD_ATEXT("\\L\\c2""    Original author, lead developer");
       ADD_ATEXT("\\L\\c0""  Stephen Anthony");
-      ADD_ATEXT("\\L\\c2""    Lead developer, Linux/Win32 maintainer");
+      ADD_ATEXT("\\L\\c2""    Lead developer, Linux/OSX/Win32 maintainer");
       ADD_ATEXT("\\L\\c0""  Mark Grebe");
-      ADD_ATEXT("\\L\\c2""    Author/maintainer for OSX port");
+      ADD_ATEXT("\\L\\c2""    Original author/maintainer for OSX port");
       ADD_ATEXT("\\L\\c0""  Brian Watson");
       ADD_ATEXT("\\L\\c2""    Emulation core enhancement, debugger support");
-      ADD_ATEXT("\\L\\c0""  Eckhard Stolberg");
-      ADD_ATEXT("\\L\\c2""    Emulation core development");
       break;
 
     case 3:
       title = "The Stella team";
-      ADD_ATEXT("\\L\\c0""  Joe D'Andrea");
-      ADD_ATEXT("\\L\\c2""    Maintainer for Solaris port");
-      ADD_ATEXT("\\L\\c0""  Darrell Spice Jr. & Doodle");
-      ADD_ATEXT("\\L\\c2""    Authors for OS/2 port");
-      ADD_ATEXT("\\L\\c0""  David Voswinkel");
-      ADD_ATEXT("\\L\\c2""    Maintainer for PSP port");
+      ADD_ATEXT("\\L\\c0""  Eckhard Stolberg");
+      ADD_ATEXT("\\L\\c2""    Emulation core development");
       ADD_ATEXT("\\L\\c0""  Kostas Nakos");
       ADD_ATEXT("\\L\\c2""    Author/maintainer for WinCE port");
-      ADD_ATEXT("\\L\\c0""  Alex Zaballa");
-      ADD_ATEXT("\\L\\c2""    Maintainer for GP2X port");
       break;
 
     case 4:
-      title = "Retired members / Contributors";
-      ADD_ATEXT("\\L\\c0""See Stella manual for contribution details");
+      title = "Contributors";
+      ADD_ATEXT("\\L\\c0""  See Stella manual for contribution details");
+      ADD_ATEXT("\\L\\c0""  and for many other people not listed here");
       ADD_ALINE;
-      ADD_ATEXT("\\L\\c0""  David Aspell, Chris Bennett, Alexander Bilstein");
-      ADD_ATEXT("\\L\\c0""  Dan Boris, Piero Cavina, Bob Colbert");
-      ADD_ATEXT("\\L\\c0""  Renato Ferreira, Ron Fries, Aaron Giles");
-      ADD_ATEXT("\\L\\c0""  Mark Hahn, Kevin Horton, Thomas Jentzsch");
-      ADD_ATEXT("\\L\\c0""  Erik \"Voch\" Kovach, Daniel Marks, James Mcclain");
-      ADD_ATEXT("\\L\\c0""  David McEwen, Jeff Miller, Dan Mowczan");
-      ADD_ATEXT("\\L\\c0""  Jack Nutting, Manuel Polik, Jim Pragit");
-      ADD_ATEXT("\\L\\c0""  John Saeger, Chris Salomon, Jason Scott");
+      ADD_ATEXT("\\L\\c0""  Thanks to the ScummVM project for the GUI code");
+      ADD_ALINE;
+      ADD_ATEXT("\\L\\c0""  Thanks to Ian Bogost and the Georgia Tech");
+      ADD_ATEXT("\\L\\c0""  Atari Team for the CRT Simulation effects");
       break;
 
     case 5:
-      title = "Retired members / Contributors";
-      ADD_ATEXT("\\L\\c0""See Stella manual for contribution details");
-      ADD_ALINE;
-      ADD_ATEXT("\\L\\c0""  David Shaw, Raul Silva, Chris Snell, John Stiles");
-      ADD_ATEXT("\\L\\c0""  Matthew Stroup, Joel Sutton, Greg Troutman");
-      ADD_ATEXT("\\L\\c0""  Curt Vendel, Keith Wilkins, Jeff Wisnia");
-      ADD_ALINE;
-      ADD_ATEXT("\\L\\c0""And many others ...");
-      ADD_ALINE;
-      ADD_ATEXT("\\L\\c0""Thanks to the ScummVM project for the GUI code");
-      break;
-
-    case 6:
       title = "Cast of thousands";
       ADD_ATEXT("\\L\\c0""Special thanks to AtariAge for introducing the");
       ADD_ATEXT("\\L\\c0""Atari 2600 to a whole new generation");
@@ -173,16 +167,15 @@ void AboutDialog::updateStrings(int page, int lines, string& title, string* &dsc
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AboutDialog::displayInfo()
 {
-  string titleStr, *dscStr;
-
-  updateStrings(myPage, LINES_PER_PAGE, titleStr, dscStr);
+  string titleStr;
+  updateStrings(myPage, kLINES_PER_PAGE, titleStr);
 
   myTitle->setLabel(titleStr);
-  for(int i = 0; i < LINES_PER_PAGE; i++)
+  for(int i = 0; i < kLINES_PER_PAGE; i++)
   {
-    const char *str = dscStr[i].c_str();
+    const char* str = myDescStr[i].c_str();
     TextAlignment align = kTextAlignCenter;
-    int color  = kTextColor;
+    uInt32 color  = kTextColor;
 
     while (str[0] == '\\')
     {
@@ -234,12 +227,10 @@ void AboutDialog::displayInfo()
     }
 
     myDesc[i]->setAlign(align);
-    myDesc[i]->setColor(color);
+    myDesc[i]->setTextColor(color);
     myDesc[i]->setLabel(str);
   }
 
-  delete[] dscStr;
-
   // Redraw entire dialog
   _dirty = true;
 }
