@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AudioDialog.cxx,v 1.24 2007/01/01 18:04:52 stephena Exp $
+// $Id: AudioDialog.cxx 1724 2009-05-13 13:55:40Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,80 +21,88 @@
 
 #include <sstream>
 
-#include "OSystem.hxx"
-#include "Sound.hxx"
-#include "Settings.hxx"
-#include "Menu.hxx"
-#include "Control.hxx"
-#include "Widget.hxx"
-#include "PopUpWidget.hxx"
-#include "Dialog.hxx"
-#include "AudioDialog.hxx"
-#include "GuiUtils.hxx"
-
 #include "bspf.hxx"
 
+#include "Console.hxx"
+#include "Control.hxx"
+#include "Dialog.hxx"
+#include "Menu.hxx"
+#include "OSystem.hxx"
+#include "PopUpWidget.hxx"
+#include "StringList.hxx"
+#include "Settings.hxx"
+#include "Sound.hxx"
+#include "Widget.hxx"
+
+#include "AudioDialog.hxx"
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AudioDialog::AudioDialog(OSystem* osystem, DialogContainer* parent,
-                         const GUI::Font& font, int x, int y, int w, int h)
-    : Dialog(osystem, parent, x, y, w, h)
+                         const GUI::Font& font)
+  : Dialog(osystem, parent, 0, 0, 0, 0)
 {
-  const int lineHeight = font.getLineHeight(),
-            fontHeight = font.getFontHeight();
+  const int lineHeight   = font.getLineHeight(),
+            fontWidth    = font.getMaxCharWidth(),
+            fontHeight   = font.getFontHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 20,
+            buttonHeight = font.getLineHeight() + 4;
   int xpos, ypos;
   int lwidth = font.getStringWidth("Fragment Size: "),
       pwidth = font.getStringWidth("4096");
   WidgetArray wid;
+  StringMap items;
+
+  // Set real dimensions
+  _w = 35 * fontWidth + 10;
+  _h = 8 * (lineHeight + 4) + 10;
 
   // Volume
-  xpos = (w - lwidth - pwidth - 40) / 2;  ypos = 10;
+  xpos = 3 * fontWidth;  ypos = 10;
 
-  myVolumeSlider = new SliderWidget(this, font, xpos, ypos, 30, lineHeight,
+  myVolumeSlider = new SliderWidget(this, font, xpos, ypos, 6*fontWidth, lineHeight,
                                     "Volume: ", lwidth, kVolumeChanged);
   myVolumeSlider->setMinValue(1); myVolumeSlider->setMaxValue(100);
   wid.push_back(myVolumeSlider);
   myVolumeLabel = new StaticTextWidget(this, font,
                                        xpos + myVolumeSlider->getWidth() + 4,
                                        ypos + 1,
-                                       15, fontHeight, "", kTextAlignLeft);
+                                       3*fontWidth, fontHeight, "", kTextAlignLeft);
 
   myVolumeLabel->setFlags(WIDGET_CLEARBG);
   ypos += lineHeight + 4;
 
   // Fragment size
+  items.clear();
+  items.push_back("128", "128");
+  items.push_back("256", "256");
+  items.push_back("512", "512");
+  items.push_back("1024", "1024");
+  items.push_back("2048", "2048");
+  items.push_back("4096", "4096");
   myFragsizePopup = new PopUpWidget(this, font, xpos, ypos,
                                     pwidth + myVolumeLabel->getWidth() - 4, lineHeight,
-                                    "Fragment size: ", lwidth);
-  myFragsizePopup->appendEntry("128",  1);
-  myFragsizePopup->appendEntry("256",  2);
-  myFragsizePopup->appendEntry("512",  3);
-  myFragsizePopup->appendEntry("1024", 4);
-  myFragsizePopup->appendEntry("2048", 5);
-  myFragsizePopup->appendEntry("4096", 6);
+                                    items, "Fragment size: ", lwidth);
   wid.push_back(myFragsizePopup);
   ypos += lineHeight + 4;
 
   // Output frequency
+  items.clear();
+  items.push_back("11025", "11025");
+  items.push_back("22050", "22050");
+  items.push_back("31400", "31400");
+  items.push_back("44100", "44100");
+  items.push_back("48000", "48000");
   myFreqPopup = new PopUpWidget(this, font, xpos, ypos,
                                 pwidth + myVolumeLabel->getWidth() - 4, lineHeight,
-                                "Output freq: ", lwidth);
-  myFreqPopup->appendEntry("11025", 1);
-  myFreqPopup->appendEntry("22050", 2);
-  myFreqPopup->appendEntry("31400", 3);
-  myFreqPopup->appendEntry("44100", 4);
-  myFreqPopup->appendEntry("48000", 5);
+                                items, "Output freq: ", lwidth);
   wid.push_back(myFreqPopup);
   ypos += lineHeight + 4;
 
   // TIA frequency
+  // ... use same items as above
   myTiaFreqPopup = new PopUpWidget(this, font, xpos, ypos,
                                    pwidth + myVolumeLabel->getWidth() - 4, lineHeight,
-                                   "TIA freq: ", lwidth);
-  myTiaFreqPopup->appendEntry("11025", 1);
-  myTiaFreqPopup->appendEntry("22050", 2);
-  myTiaFreqPopup->appendEntry("31400", 3);
-  myTiaFreqPopup->appendEntry("44100", 4);
-  myTiaFreqPopup->appendEntry("48000", 5);
+                                   items, "TIA freq: ", lwidth);
   wid.push_back(myTiaFreqPopup);
   ypos += lineHeight + 4;
 
@@ -112,23 +120,10 @@ AudioDialog::AudioDialog(OSystem* osystem, DialogContainer* parent,
 
   // Add Defaults, OK and Cancel buttons
   ButtonWidget* b;
-  b = addButton(font, 10, _h - 24, "Defaults", kDefaultsCmd);
+  b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
   wid.push_back(b);
-#ifndef MAC_OSX
-  b = addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "OK", kOKCmd);
-  wid.push_back(b);
-  addOKWidget(b);
-  b = addButton(font, _w - (kButtonWidth + 10), _h - 24, "Cancel", kCloseCmd);
-  wid.push_back(b);
-  addCancelWidget(b);
-#else
-  b = addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "Cancel", kCloseCmd);
-  wid.push_back(b);
-  addCancelWidget(b);
-  b = addButton(font, _w - (kButtonWidth + 10), _h - 24, "OK", kOKCmd);
-  wid.push_back(b);
-  addOKWidget(b);
-#endif
+  addOKCancelBGroup(wid, font);
 
   addToFocusList(wid);
 }
@@ -141,49 +136,24 @@ AudioDialog::~AudioDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AudioDialog::loadConfig()
 {
-  bool b;
-  int i;
-
   // Volume
-  myVolumeSlider->setValue(instance()->settings().getInt("volume"));
-  myVolumeLabel->setLabel(instance()->settings().getString("volume"));
+  myVolumeSlider->setValue(instance().settings().getInt("volume"));
+  myVolumeLabel->setLabel(instance().settings().getString("volume"));
 
   // Fragsize
-  i = instance()->settings().getInt("fragsize");
-  if(i == 128)       i = 1;
-  else if(i == 256)  i = 2;
-  else if(i == 512)  i = 3;
-  else if(i == 1024) i = 4;
-  else if(i == 2048) i = 5;
-  else if(i == 4096) i = 6;
-  myFragsizePopup->setSelectedTag(i);
+  myFragsizePopup->setSelected(instance().settings().getString("fragsize"), "512");
 
   // Output frequency
-  i = instance()->settings().getInt("freq");
-  if(i == 11025)      i = 1;
-  else if(i == 22050) i = 2;
-  else if(i == 31400) i = 3;
-  else if(i == 44100) i = 4;
-  else if(i == 48000) i = 5;
-  else i = 3;  // default to '31400'
-  myFreqPopup->setSelectedTag(i);
+  myFreqPopup->setSelected(instance().settings().getString("freq"), "31400");
 
   // TIA frequency
-  i = instance()->settings().getInt("tiafreq");
-  if(i == 11025)      i = 1;
-  else if(i == 22050) i = 2;
-  else if(i == 31400) i = 3;
-  else if(i == 44100) i = 4;
-  else if(i == 48000) i = 5;
-  else i = 3;  // default to '31400'
-  myTiaFreqPopup->setSelectedTag(i);
+  myTiaFreqPopup->setSelected(instance().settings().getString("tiafreq"), "31400");
 
   // Clip volume
-  b = instance()->settings().getBool("clipvol");
-  myClipVolumeCheckbox->setState(b);
+  myClipVolumeCheckbox->setState(instance().settings().getBool("clipvol"));
 
   // Enable sound
-  b = instance()->settings().getBool("sound");
+  bool b = instance().settings().getBool("sound");
   mySoundEnableCheckbox->setState(b);
 
   // Make sure that mutually-exclusive items are not enabled at the same time
@@ -193,39 +163,31 @@ void AudioDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AudioDialog::saveConfig()
 {
-  Settings& settings = instance()->settings();
-  string s;
-  bool b;
-  int i;
+  Settings& settings = instance().settings();
 
   // Volume
-  i = myVolumeSlider->getValue();
-  instance()->sound().setVolume(i);
+  settings.setInt("volume", myVolumeSlider->getValue());
+  instance().sound().setVolume(myVolumeSlider->getValue());
 
   // Fragsize
-  s = myFragsizePopup->getSelectedString();
-  settings.setString("fragsize", s);
+  settings.setString("fragsize", myFragsizePopup->getSelectedTag());
 
   // Output frequency
-  s = myFreqPopup->getSelectedString();
-  settings.setString("freq", s);
+  settings.setString("freq", myFreqPopup->getSelectedTag());
 
   // TIA frequency
-  s = myTiaFreqPopup->getSelectedString();
-  settings.setString("tiafreq", s);
+  settings.setString("tiafreq", myTiaFreqPopup->getSelectedTag());
 
   // Enable/disable volume clipping (requires a restart to take effect)
-  b = myClipVolumeCheckbox->getState();
-  settings.setBool("clipvol", b);
+  settings.setBool("clipvol", myClipVolumeCheckbox->getState());
 
   // Enable/disable sound (requires a restart to take effect)
-  b = mySoundEnableCheckbox->getState();
-  instance()->sound().setEnabled(b);
+  instance().sound().setEnabled(mySoundEnableCheckbox->getState());
 
   // Only force a re-initialization when necessary, since it can
   // be a time-consuming operation
-  if(&instance()->console())
-    instance()->console().initializeAudio();
+  if(&instance().console())
+    instance().console().initializeAudio();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -234,13 +196,9 @@ void AudioDialog::setDefaults()
   myVolumeSlider->setValue(100);
   myVolumeLabel->setLabel("100");
 
-#ifdef WIN32
-  myFragsizePopup->setSelectedTag(5);
-#else
-  myFragsizePopup->setSelectedTag(3);
-#endif
-  myFreqPopup->setSelectedTag(3);
-  myTiaFreqPopup->setSelectedTag(3);
+  myFragsizePopup->setSelected("512", "");
+  myFreqPopup->setSelected("31400", "");
+  myTiaFreqPopup->setSelected("31400", "");
 
   myClipVolumeCheckbox->setState(true);
   mySoundEnableCheckbox->setState(true);
