@@ -8,51 +8,56 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: BrowserDialog.cxx,v 1.24 2007/01/01 18:04:52 stephena Exp $
+// $Id: BrowserDialog.cxx 1836 2009-07-12 22:09:21Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "Widget.hxx"
-#include "StringListWidget.hxx"
-#include "Dialog.hxx"
-#include "FSNode.hxx"
-#include "GuiObject.hxx"
-#include "GuiUtils.hxx"
-#include "GameList.hxx"
-#include "BrowserDialog.hxx"
-
 #include "bspf.hxx"
 
+#include "Dialog.hxx"
+#include "DialogContainer.hxx"
+#include "FSNode.hxx"
+#include "GameList.hxx"
+#include "GuiObject.hxx"
+#include "OSystem.hxx"
+#include "StringListWidget.hxx"
+#include "Widget.hxx"
+
+#include "BrowserDialog.hxx"
+
 /* We want to use this as a general directory selector at some point... possible uses
  * - to select the data dir for a game
  * - to select the place where save games are stored
  * - others???
- * TODO - make this dialog font sensitive
  */
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
-                             int x, int y, int w, int h)
-  : Dialog(boss->instance(), boss->parent(), x, y, w, h),
+BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font)
+  : Dialog(&boss->instance(), &boss->parent(), 0, 0, 0, 0),
     CommandSender(boss),
     _fileList(NULL),
     _currentPath(NULL),
-    _nodeList(NULL)
+    _nodeList(NULL),
+    _mode(FilesystemNode::kListDirectoriesOnly)
 {
-  const int lineHeight = font.getLineHeight(),
-            bwidth     = font.getStringWidth("Cancel") + 20,
-            bheight    = font.getLineHeight() + 4;
+  const int lineHeight   = font.getLineHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 20,
+            buttonHeight = font.getLineHeight() + 4;
   int xpos, ypos;
   ButtonWidget* b;
 
+  // Set real dimensions
+  // This is one dialog that can take as much space as is available
+  _w = BSPF_min(instance().desktopWidth(), 480u);
+  _h = BSPF_min(instance().desktopHeight(), 380u);
+
   xpos = 10;  ypos = 4;
   _title = new StaticTextWidget(this, font, xpos, ypos,
                                 _w - 2 * xpos, lineHeight,
@@ -67,35 +72,36 @@ BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
   // Add file list
   ypos += lineHeight;
   _fileList = new StringListWidget(this, font, xpos, ypos,
-                                   _w - 2 * xpos, _h - bheight - ypos - 15);
+                                   _w - 2 * xpos, _h - buttonHeight - ypos - 20);
   _fileList->setNumberingMode(kListNumberingOff);
   _fileList->setEditable(false);
 
   // Buttons
-  xpos = 10;  ypos = _h - bheight - 8;
-  b = _goUpButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
-                                     "Go up", kGoUpCmd);
-  addFocusWidget(b);
+  _goUpButton = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
+                                 buttonWidth, buttonHeight, "Go up", kGoUpCmd);
+  addFocusWidget(_goUpButton);
+
+  _basedirButton =
+    new ButtonWidget(this, font, 15 + buttonWidth, _h - buttonHeight - 10,
+                     buttonWidth, buttonHeight, "Base Dir", kBaseDirCmd);
+  addFocusWidget(_basedirButton);
+
 #ifndef MAC_OSX
-  xpos = _w - 2 *(bwidth + 10);  
-  b = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Choose",
-                       kChooseCmd);
+  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Choose", kChooseCmd);
   addFocusWidget(b);
   addOKWidget(b);
-  xpos += bwidth + 10;
-  b = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Cancel",
-                       kCloseCmd);
+  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
   addFocusWidget(b);
   addCancelWidget(b);
 #else
-  xpos = _w - 2 *(bwidth + 10);  ypos = _h - bheight - 8;
-  b = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Cancel",
-                       kCloseCmd);
+  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
   addFocusWidget(b);
   addCancelWidget(b);
-  xpos += bwidth + 10;
-  b = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Choose",
-                       kChooseCmd);
+  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Choose", kChooseCmd);
   addFocusWidget(b);
   addOKWidget(b);
 #endif
@@ -113,14 +119,34 @@ BrowserDialog::~BrowserDialog()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void BrowserDialog::setStartPath(const string& startpath)
+void BrowserDialog::show(const string& title, const string& startpath,
+                         FilesystemNode::ListMode mode, int cmd)
 {
-  // If no node has been set, or the last used one is now invalid,
-  // go back to the root/default dir.
-  _node = FilesystemNode(startpath);
+  // TODO - dialog has to be added before any settings are changed,
+  //        since (for example) changing the title triggers a redraw,
+  //        and the dialog must be added (so it exists) for that to happen
+  // Fixing this requires changes to the underlying widget classes
+  // (ie, changing a widgets contents should signal its dialog that a
+  // redraw is necessary; it shouldn't be responsible for redraw itself)
+  //
+  // Doing it this way has the unfortunate side effect that a previous
+  // title is temporarily visible when re-using the browser for different
+  // purposes
+  parent().addDialog(this);
 
-  if(!_node.isValid())
-    _node = FilesystemNode();
+  _title->setLabel(title);
+  _cmd = cmd;
+  _mode = mode;
+
+  // If no node has been set, or the last used one is now invalid,
+  // go back to the users home dir.
+  _node = FilesystemNode(startpath);
+  if(!_node.exists())
+    _node = FilesystemNode("~");
+
+  // Generally, we always want a directory listing 
+  if(!_node.isDirectory() && _node.hasParent())
+    _node = _node.getParent();
 
   // Alway refresh file list
   updateListing();
@@ -136,29 +162,27 @@ void BrowserDialog::updateListing()
   _nodeList->clear();
 
   // Update the path display
-  _currentPath->setLabel(_node.path());
+  _currentPath->setLabel(_node.getRelativePath());
 
   // Read in the data from the file system
-  FSList content = _node.listDir();
-  content.sort();
+  FSList content;
+  _node.getChildren(content, _mode);
 
   // Add '[..]' to indicate previous folder
   if(_node.hasParent())
-  {
-    const string& parent = _node.getParent().path();
-    _nodeList->appendGame(" [..]", parent, "", true);
-  }
+    _nodeList->appendGame(" [..]", _node.getParent().getPath(), "", true);
 
   // Now add the directory entries
   for(unsigned int idx = 0; idx < content.size(); idx++)
   {
-    string name = content[idx].displayName();
+    string name = content[idx].getDisplayName();
     bool isDir = content[idx].isDirectory();
     if(isDir)
       name = " [" + name + "]";
 
-    _nodeList->appendGame(name, content[idx].path(), "", isDir);
+    _nodeList->appendGame(name, content[idx].getPath(), "", isDir);
   }
+  _nodeList->sortByName();
 
   // Now fill the list widget with the contents of the GameList
   StringList l;
@@ -171,9 +195,6 @@ void BrowserDialog::updateListing()
 
   // Only hilite the 'up' button if there's a parent directory
   _goUpButton->setEnabled(_node.hasParent());
-
-  // Finally, redraw
-  setDirty(); draw();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -190,17 +211,23 @@ void BrowserDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kGoUpCmd:
+    case kListPrevDirCmd:
       _node = _node.getParent();
       updateListing();
       break;
 
+    case kBaseDirCmd:
+      _node = FilesystemNode(instance().baseDir());
+      updateListing();
+      break;
+
     case kListItemActivatedCmd:
     case kListItemDoubleClickedCmd:
     {
       int item = _fileList->getSelected();
       if(item >= 0)
       {
-        _node = _nodeList->path(item);
+        _node = FilesystemNode(_nodeList->path(item));
         updateListing();
       }
       break;
