@@ -8,29 +8,27 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DialogContainer.cxx,v 1.36 2007/01/01 18:04:52 stephena Exp $
+// $Id: DialogContainer.cxx 1823 2009-07-01 14:39:01Z stephena $
 //============================================================================
 
 #include "OSystem.hxx"
 #include "Dialog.hxx"
 #include "Stack.hxx"
 #include "EventHandler.hxx"
+#include "Joystick.hxx"
 #include "bspf.hxx"
 #include "DialogContainer.hxx"
 
-#define JOY_DEADZONE 3200
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DialogContainer::DialogContainer(OSystem* osystem)
   : myOSystem(osystem),
     myBaseDialog(NULL),
-    myTime(0),
-    myRefreshFlag(false)
+    myTime(0)
 {
   reset();
 }
@@ -43,7 +41,7 @@ DialogContainer::~DialogContainer()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::updateTime(uInt32 time)
+void DialogContainer::updateTime(uInt64 time)
 {
   // We only need millisecond precision
   myTime = time / 1000;
@@ -88,17 +86,17 @@ void DialogContainer::updateTime(uInt32 time)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::draw()
+void DialogContainer::draw(bool full)
 {
   // Draw all the dialogs on the stack when we want a full refresh
-  if(myRefreshFlag)
+  if(full)
   {
     for(int i = 0; i < myDialogStack.size(); i++)
     {
+      myDialogStack[i]->center();
       myDialogStack[i]->setDirty();
       myDialogStack[i]->drawDialog();
     }
-    myRefreshFlag = false;
   }
   else if(!myDialogStack.empty())
   {
@@ -110,10 +108,9 @@ void DialogContainer::draw()
 void DialogContainer::addDialog(Dialog* d)
 {
   myDialogStack.push(d);
-
-  d->center();
   d->open();
-  d->setDirty();  // Next update() will take care of drawing
+
+  myOSystem->frameBuffer().refresh();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -125,7 +122,7 @@ void DialogContainer::removeDialog()
 
     // We need to redraw the entire screen contents, since we don't know
     // what was obscured
-    myOSystem->eventHandler().refreshDisplay();
+    myOSystem->frameBuffer().refresh();
   }
 }
 
@@ -137,9 +134,6 @@ void DialogContainer::reStack()
     myDialogStack.pop();
   addDialog(myBaseDialog);
 
-  // Erase any previous messages
-  myOSystem->frameBuffer().hideMessage();
-
   // Reset all continuous events
   reset();
 }
@@ -179,6 +173,7 @@ void DialogContainer::handleMouseMotionEvent(int x, int y, int button)
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
+  activeDialog->surface().translateCoords(x, y);
   activeDialog->handleMouseMoved(x - activeDialog->_x,
                                  y - activeDialog->_y,
                                  button);
@@ -195,6 +190,7 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, int x, int y, uInt8
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
+  activeDialog->surface().translateCoords(x, y);
 
   int button = (b == EVENT_LBUTTONDOWN || b == EVENT_LBUTTONUP) ? 1 : 2;
   switch(b)
@@ -210,8 +206,8 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, int x, int y, uInt8
       }
 
       if(myLastClick.count && (myTime < myLastClick.time + kDoubleClickDelay)
-         && ABS(myLastClick.x - x) < 3
-         && ABS(myLastClick.y - y) < 3)
+         && BSPF_abs(myLastClick.x - x) < 3
+         && BSPF_abs(myLastClick.y - y) < 3)
       {
         myLastClick.count++;
       }
@@ -282,16 +278,19 @@ void DialogContainer::handleJoyEvent(int stick, int button, uInt8 state)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::handleJoyAxisEvent(int stick, int axis, int value)
 {
+// FIXME - analog axis events cause autofire to inadvertently come on and not go off
+
   if(myDialogStack.empty())
     return;
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
 
-  if(value > JOY_DEADZONE)
-    value -= JOY_DEADZONE;
-  else if(value < -JOY_DEADZONE )
-    value += JOY_DEADZONE;
+  int deadzone = Joystick::deadzone();
+  if(value > deadzone)
+    value -= deadzone;
+  else if(value < -deadzone )
+    value += deadzone;
   else
     value = 0;
 
