@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EditableWidget.cxx,v 1.21 2007/01/01 18:04:52 stephena Exp $
+// $Id: EditableWidget.cxx 1849 2009-08-05 16:05:34Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
+#include "Dialog.hxx"
 #include "EditableWidget.hxx"
 
 
@@ -36,6 +37,11 @@ EditableWidget::EditableWidget(GuiObject* boss, const GUI::Font& font,
   _caretInverse = false;
 
   _editScrollOffset = 0;
+
+  _bgcolor = kWidColor;
+  _bgcolorhi = kWidColor;
+  _textcolor = kTextColor;
+  _textcolorhi = kTextColor;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -71,7 +77,7 @@ void EditableWidget::setEditable(bool editable)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool EditableWidget::tryInsertChar(char c, int pos)
 {
-  if (isprint(c))
+  if(isprint(c) && c != '\"')
   {
     _editString.insert(pos, 1, c);
     return true;
@@ -86,13 +92,13 @@ bool EditableWidget::handleKeyDown(int ascii, int keycode, int modifiers)
     return true;
 
   // Ignore all alt-mod keys
-  if(instance()->eventHandler().kbdAlt(modifiers))
+  if(instance().eventHandler().kbdAlt(modifiers))
     return true;
 
   bool handled = true;
   bool dirty = false;
 
-  switch (keycode)
+  switch (ascii)
   {
     case '\n':  // enter/return
     case '\r':
@@ -110,19 +116,25 @@ bool EditableWidget::handleKeyDown(int ascii, int keycode, int modifiers)
 
     case 8:     // backspace
       dirty = killChar(-1);
+      if(dirty)  sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 127:   // delete
       dirty = killChar(+1);
+      if(dirty)  sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 256 + 20:  // left arrow
-		if (_caretPos > 0)
-          dirty = setCaretPos(_caretPos - 1);
-        break;
+      if(instance().eventHandler().kbdControl(modifiers))
+        dirty = specialKeys(ascii, keycode);
+      else if(_caretPos > 0)
+        dirty = setCaretPos(_caretPos - 1);
+      break;
 
     case 256 + 19:  // right arrow
-      if (_caretPos < (int)_editString.size())
+      if(instance().eventHandler().kbdControl(modifiers))
+        dirty = specialKeys(ascii, keycode);
+      else if(_caretPos < (int)_editString.size())
         dirty = setCaretPos(_caretPos + 1);
       break;
 
@@ -135,9 +147,9 @@ bool EditableWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     default:
-      if (instance()->eventHandler().kbdControl(modifiers))
+      if (instance().eventHandler().kbdControl(modifiers))
       {
-        dirty = specialKeys(keycode);
+        dirty = specialKeys(ascii, keycode);
       }
       else if (tryInsertChar((char)ascii, _caretPos))
       {
@@ -177,9 +189,7 @@ void EditableWidget::drawCaret()
   if (!_editable || !isVisible() || !_boss->isVisible() || !_hasFocus)
     return;
 
-  GUI::Rect editRect = getEditRect();
-
-  int color = kTextColorHi;
+  const GUI::Rect& editRect = getEditRect();
   int x = editRect.left;
   int y = editRect.top;
 
@@ -188,8 +198,8 @@ void EditableWidget::drawCaret()
   x += _x;
   y += _y;
 
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
-  fb.vLine(x, y+2, y + editRect.height() - 2, color);
+  FBSurface& s = _boss->dialog().surface();
+  s.vLine(x, y+2, y + editRect.height() - 3, kTextColorHi);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -239,7 +249,7 @@ bool EditableWidget::adjustOffset()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool EditableWidget::specialKeys(int keycode)
+bool EditableWidget::specialKeys(int ascii, int keycode)
 {
   bool handled = true;
 
@@ -249,24 +259,46 @@ bool EditableWidget::specialKeys(int keycode)
       setCaretPos(0);
       break;
 
+    case 'c':
+      copySelectedText();
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
+      break;
+
     case 'e':
       setCaretPos(_editString.size());
       break;
 
     case 'd':
       handled = killChar(+1);
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'k':
       handled = killLine(+1);
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'u':
       handled = killLine(-1);
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
+      break;
+
+    case 'v':
+      pasteSelectedText();
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'w':
       handled = killLastWord();
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
+      break;
+
+    case 256 + 20:  // left arrow
+      handled = moveWord(-1);
+      break;
+
+    case 256 + 19:  // right arrow
+      handled = moveWord(+1);
       break;
 
     default:
@@ -360,3 +392,63 @@ bool EditableWidget::killLastWord()
 
   return handled;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool EditableWidget::moveWord(int direction)
+{
+  bool handled = false;
+  bool space = true;
+  int currentPos = _caretPos;
+
+  if(direction == -1)  // move to first character of previous word
+  {
+    while (currentPos > 0)
+    {
+      if (_editString[currentPos - 1] == ' ')
+      {
+        if (!space)
+          break;
+      }
+      else
+        space = false;
+
+      currentPos--;
+    }
+    _caretPos = currentPos;
+    handled = true;
+  }
+  else if(direction == +1)  // move to first character of next word
+  {
+    while (currentPos < (int)_editString.size())
+    {
+      if (_editString[currentPos - 1] == ' ')
+      {
+        if (!space)
+          break;
+      }
+      else
+        space = false;
+
+      currentPos++;
+    }
+    _caretPos = currentPos;
+    handled = true;
+  }
+
+  return handled;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EditableWidget::copySelectedText()
+{
+  _clippedText = _editString;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EditableWidget::pasteSelectedText()
+{
+  _editString = _clippedText;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string EditableWidget::_clippedText = "";
