@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EventMappingWidget.cxx,v 1.19 2007/01/01 18:04:52 stephena Exp $
+// $Id: EventMappingWidget.cxx 1724 2009-05-13 13:55:40Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,16 +21,16 @@
 
 #include <sstream>
 
-#include "OSystem.hxx"
-#include "Widget.hxx"
-#include "StringListWidget.hxx"
-#include "GuiUtils.hxx"
-#include "Event.hxx"
-#include "EventHandler.hxx"
-#include "EventMappingWidget.hxx"
-
 #include "bspf.hxx"
 
+#include "EventHandler.hxx"
+#include "Event.hxx"
+#include "OSystem.hxx"
+#include "StringListWidget.hxx"
+#include "Widget.hxx"
+
+#include "EventMappingWidget.hxx"
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
                                        int x, int y, int w, int h,
@@ -42,12 +42,14 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
     myRemapStatus(false),
     myFirstTime(true)
 {
-  const int fontHeight = font.getFontHeight(),
-            lineHeight = font.getLineHeight();
+  const int fontHeight   = font.getFontHeight(),
+            lineHeight   = font.getLineHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 10,
+            buttonHeight = font.getLineHeight() + 4;
   int xpos = 5, ypos = 5;
 
   myActionsList = new StringListWidget(boss, font, xpos, ypos,
-                                       _w - 70, _h - 3*lineHeight);
+                                       _w - buttonWidth - 20, _h - 3*lineHeight);
   myActionsList->setTarget(this);
   myActionsList->setNumberingMode(kListNumberingOff);
   myActionsList->setEditable(false);
@@ -55,24 +57,28 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
   addFocusWidget(myActionsList);
 
   // Add remap, erase, cancel and default buttons
-  xpos += myActionsList->getWidth() + 15;  ypos += 5;
-  myMapButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  xpos += myActionsList->getWidth() + 5;  ypos += 5;
+  myMapButton = new ButtonWidget(boss, font, xpos, ypos,
+                                 buttonWidth, buttonHeight,
                                  "Map", kStartMapCmd);
   myMapButton->setTarget(this);
   addFocusWidget(myMapButton);
-  ypos += 20;
-  myEraseButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  ypos += lineHeight + 10;
+  myEraseButton = new ButtonWidget(boss, font, xpos, ypos,
+                                   buttonWidth, buttonHeight,
                                    "Erase", kEraseCmd);
   myEraseButton->setTarget(this);
   addFocusWidget(myEraseButton);
-  ypos += 20;
-  myCancelMapButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  ypos += lineHeight + 10;
+  myCancelMapButton = new ButtonWidget(boss, font, xpos, ypos,
+                                       buttonWidth, buttonHeight,
                                        "Cancel", kStopMapCmd);
   myCancelMapButton->setTarget(this);
   myCancelMapButton->clearFlags(WIDGET_ENABLED);
   addFocusWidget(myCancelMapButton);
-  ypos += 30;
-  myDefaultsButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  ypos += lineHeight + 30;
+  myDefaultsButton = new ButtonWidget(boss, font, xpos, ypos,
+                                      buttonWidth, buttonHeight,
                                       "Defaults", kDefaultsCmd);
   myDefaultsButton->setTarget(this);
   addFocusWidget(myDefaultsButton);
@@ -127,9 +133,9 @@ void EventMappingWidget::startRemapping()
   // And show a message indicating which key is being remapped
   ostringstream buf;
   buf << "Select action for '"
-      << instance()->eventHandler().actionAtIndex(myActionSelected, myEventMode)
+      << instance().eventHandler().actionAtIndex(myActionSelected, myEventMode)
       << "' event";	 	
-  myKeyMapping->setColor(kTextColorEm);
+  myKeyMapping->setTextColor(kTextColorEm);
   myKeyMapping->setLabel(buf.str());
 
   // Make sure that this widget receives all raw data, before any
@@ -144,8 +150,8 @@ void EventMappingWidget::eraseRemapping()
     return;
 
   Event::Type event =
-    instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-  instance()->eventHandler().eraseMapping(event, myEventMode);
+    instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+  instance().eventHandler().eraseMapping(event, myEventMode);
 
   drawKeyMapping();
 }
@@ -182,8 +188,8 @@ void EventMappingWidget::drawKeyMapping()
   {
     ostringstream buf;
     buf << "Action: "
-        << instance()->eventHandler().keyAtIndex(myActionSelected, myEventMode);
-    myKeyMapping->setColor(kTextColor);
+        << instance().eventHandler().keyAtIndex(myActionSelected, myEventMode);
+    myKeyMapping->setTextColor(kTextColor);
     myKeyMapping->setLabel(buf.str());
   }
 }
@@ -195,8 +201,8 @@ bool EventMappingWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addKeyMapping(event, myEventMode, keycode))
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addKeyMapping(event, myEventMode, keycode))
       stopRemapping();
   }
   return true;
@@ -209,8 +215,8 @@ void EventMappingWidget::handleJoyDown(int stick, int button)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addJoyMapping(event, myEventMode, stick, button))
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addJoyMapping(event, myEventMode, stick, button))
       stopRemapping();
   }
 }
@@ -222,8 +228,8 @@ void EventMappingWidget::handleJoyAxis(int stick, int axis, int value)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addJoyAxisMapping(event, myEventMode,
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addJoyAxisMapping(event, myEventMode,
                                                     stick, axis, value))
       stopRemapping();
   }
@@ -238,8 +244,8 @@ bool EventMappingWidget::handleJoyHat(int stick, int hat, int value)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addJoyHatMapping(event, myEventMode,
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addJoyHatMapping(event, myEventMode,
                                                    stick, hat, value))
     {
       stopRemapping();
@@ -290,7 +296,7 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kDefaultsCmd:
-      instance()->eventHandler().setDefaultMapping(myEventMode);
+      instance().eventHandler().setDefaultMapping(myEventMode);
       drawKeyMapping();
       break;
   }
