@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ProgressDialog.cxx,v 1.8 2007/01/01 18:04:54 stephena Exp $
+// $Id: ProgressDialog.cxx 1724 2009-05-13 13:55:40Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -24,19 +24,18 @@
 #include "Dialog.hxx"
 #include "DialogContainer.hxx"
 #include "ProgressDialog.hxx"
-#include "GuiUtils.hxx"
 
 #include "bspf.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
                                const string& message)
-    : Dialog(boss->instance(), boss->parent(), 0, 0, 16, 16),
-      myMessage(NULL),
-      mySlider(NULL),
-      myStart(0),
-      myFinish(0),
-      myStep(0)
+  : Dialog(&boss->instance(), &boss->parent(), 0, 0, 16, 16),
+    myMessage(NULL),
+    mySlider(NULL),
+    myStart(0),
+    myFinish(0),
+    myStep(0)
 {
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
@@ -47,23 +46,18 @@ ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
   lwidth = font.getStringWidth(message);
   _w = lwidth + 2 * fontWidth;
   _h = lineHeight * 5;
-  _x = (boss->getWidth() - _w) / 2;
-  _y = (boss->getHeight() - _h) / 2;
 
   xpos = fontWidth; ypos = lineHeight;
   myMessage = new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
                                    message, kTextAlignCenter);
-  myMessage->setColor(kTextColorEm);
+  myMessage->setTextColor(kTextColorEm);
 
   xpos = fontWidth; ypos += 2 * lineHeight;
   mySlider = new SliderWidget(this, font, xpos, ypos, lwidth, lineHeight, "", 0, 0);
-  mySlider->setMinValue(100);
-  mySlider->setMaxValue(200);
-  mySlider->setValue(100);  // Prevents the slider from initially drawing
-                            // across the entire screen for a split-second
+  mySlider->setMinValue(1);
+  mySlider->setMaxValue(100);
 
-  parent()->addDialog(this);
-  instance()->frameBuffer().update();
+  parent().addDialog(this);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -71,12 +65,6 @@ ProgressDialog::~ProgressDialog()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ProgressDialog::done()
-{
-  parent()->removeDialog();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ProgressDialog::setMessage(const string& message)
 {
@@ -88,20 +76,25 @@ void ProgressDialog::setRange(int start, int finish, int step)
 {
   myStart = start;
   myFinish = finish;
-  myStep = step;
-  myCurrentStep = 100;
+  myStep = (int)((step / 100.0) * (myFinish - myStart + 1));
+
+  mySlider->setMinValue(myStart);
+  mySlider->setMaxValue(myFinish);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ProgressDialog::setProgress(int progress)
 {
   // Only increase the progress bar if we have arrived at a new step
-  // IE, we only increase in intervals specified by setRange()
-  int p = (int) (((double)progress / myFinish) * 100 + 100);
-  if(p >= myCurrentStep)
+  if(progress - mySlider->getValue() > myStep)
   {
-    myCurrentStep += myStep;
-    mySlider->setValue(p);
-    instance()->frameBuffer().update();
+    mySlider->setValue(progress);
+
+    // Since this dialog is usually called in a tight loop that doesn't
+    // yield, we need to manually tell the framebuffer that a redraw is
+    // necessary
+    // This isn't really an ideal solution, since all redrawing and
+    // event handling is suspended until the dialog is closed
+    instance().frameBuffer().update();
   }
 }
