@@ -8,40 +8,44 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Widget.cxx,v 1.48 2007/01/15 13:51:55 stephena Exp $
+// $Id: Widget.cxx 1724 2009-05-13 13:55:40Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "FrameBuffer.hxx"
-#include "Font.hxx"
-#include "Dialog.hxx"
-#include "DialogContainer.hxx"
-#include "Command.hxx"
-#include "GuiObject.hxx"
 #include "bspf.hxx"
-#include "GuiUtils.hxx"
-#include "Widget.hxx"
+
+#include "Command.hxx"
+#include "DialogContainer.hxx"
+#include "Dialog.hxx"
 #include "EditableWidget.hxx"
+#include "Font.hxx"
+#include "FrameBuffer.hxx"
+#include "GuiObject.hxx"
+#include "OSystem.hxx"
+
+#include "Widget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Widget::Widget(GuiObject* boss, const GUI::Font& font,
                int x, int y, int w, int h)
-  : GuiObject(boss->instance(), boss->parent(), x, y, w, h),
+  : GuiObject(boss->instance(), boss->parent(), boss->dialog(), x, y, w, h),
     _type(0),
     _boss(boss),
     _font((GUI::Font*)&font),
     _id(-1),
     _flags(0),
     _hasFocus(false),
-    _color(kTextColor)
+    _bgcolor(kWidColor),
+    _bgcolorhi(kWidColor),
+    _textcolor(kTextColor),
+    _textcolorhi(kTextColorHi)
 {
   // Insert into the widget list of the boss
   _next = _boss->_firstWidget;
@@ -68,11 +72,12 @@ void Widget::draw()
 
   _dirty = false;
   
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
 
   if(!isVisible() || !_boss->isVisible())
     return;
 
+  bool hasBorder = _flags & WIDGET_BORDER;
   int oldX = _x, oldY = _y, oldW = _w, oldH = _h;
 
   // Account for our relative position in the dialog
@@ -81,15 +86,18 @@ void Widget::draw()
 
   // Clear background (unless alpha blending is enabled)
   if(_flags & WIDGET_CLEARBG)
-    fb.fillRect(_x, _y, _w, _h, kBGColor);
+  {
+    int x = _x, y = _y, w = _w, h = _h;
+    if(hasBorder)
+    {
+      x++; y++; w-=2; h-=2;
+    }
+    s.fillRect(x, y, w, h, (_flags & WIDGET_HILITED) ? _bgcolorhi : _bgcolor);
+  }
 
   // Draw border
-  if(_flags & WIDGET_BORDER) {
-    int colorA = kColor;
-    int colorB = kShadowColor;
-    if((_flags & WIDGET_INV_BORDER) == WIDGET_INV_BORDER)
-      SWAP(colorA, colorB);
-    fb.box(_x, _y, _w, _h, colorA, colorB);
+  if(hasBorder) {
+    s.box(_x, _y, _w, _h, kColor, kShadowColor);
     _x += 4;
     _y += 4;
     _w -= 8;
@@ -100,7 +108,7 @@ void Widget::draw()
   drawWidget((_flags & WIDGET_HILITED) ? true : false);
 
   // Restore x/y
-  if (_flags & WIDGET_BORDER) {
+  if (hasBorder) {
     _x -= 4;
     _y -= 4;
     _w += 8;
@@ -119,7 +127,7 @@ void Widget::draw()
   }
 
   // Tell the framebuffer this area is dirty
-  fb.addDirtyRect(getAbsX(), getAbsY(), oldW, oldH);
+  s.addDirtyRect(getAbsX(), getAbsY(), oldW, oldH);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -142,16 +150,6 @@ void Widget::lostFocus()
   lostFocusWidget();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect Widget::getRect() const
-{
-  int x = getAbsX() - 1,  y = getAbsY() - 1,
-      w = getWidth() + 2, h = getHeight() + 2;
-
-  GUI::Rect r(x, y, x+w, y+h);
-  return r;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Widget::setEnabled(bool e)
 {
@@ -220,7 +218,7 @@ bool Widget::isWidgetInChain(WidgetArray& list, Widget* find)
 Widget* Widget::setFocusForChain(GuiObject* boss, WidgetArray& arr,
                                  Widget* wid, int direction)
 {
-  FrameBuffer& fb = boss->instance()->frameBuffer();
+  FBSurface& s = boss->dialog().surface();
   int size = arr.size(), pos = -1;
   Widget* tmp;
   for(int i = 0; i < size; ++i)
@@ -231,18 +229,21 @@ Widget* Widget::setFocusForChain(GuiObject* boss, WidgetArray& arr,
     if(wid == tmp)
       pos = i;
 
-    GUI::Rect rect = tmp->getRect();
-    int x = rect.left,    y = rect.top,
-        w = rect.width(), h = rect.height();
+    // Get area around widget
+    // Note: we must use getXXX() methods and not access the variables
+    // directly, since in some cases (notably those widgets with embedded
+    // ScrollBars) the two quantities may be different
+    int x = tmp->getAbsX() - 1,  y = tmp->getAbsY() - 1,
+        w = tmp->getWidth() + 2, h = tmp->getHeight() + 2;
 
     // First clear area surrounding all widgets
     if(tmp->_hasFocus)
     {
       tmp->lostFocus();
-      fb.frameRect(x, y, w, h, kBGColor);
+      s.frameRect(x, y, w, h, kDlgColor);
 
       tmp->setDirty(); tmp->draw();
-      fb.addDirtyRect(x, y, w, h);
+      s.addDirtyRect(x, y, w, h);
     }
   }
 
@@ -273,15 +274,19 @@ Widget* Widget::setFocusForChain(GuiObject* boss, WidgetArray& arr,
 
   // Now highlight the active widget
   tmp = arr[pos];
-  GUI::Rect rect = tmp->getRect();
-  int x = rect.left,    y = rect.top,
-      w = rect.width(), h = rect.height();
+
+  // Get area around widget
+  // Note: we must use getXXX() methods and not access the variables
+  // directly, since in some cases (notably those widgets with embedded
+  // ScrollBars) the two quantities may be different
+  int x = tmp->getAbsX() - 1,  y = tmp->getAbsY() - 1,
+      w = tmp->getWidth() + 2, h = tmp->getHeight() + 2;
 
   tmp->receivedFocus();
-  fb.frameRect(x, y, w, h, kTextColorEm, kDashLine);
+  s.frameRect(x, y, w, h, kWidFrameColor, kDashLine);
 
   tmp->setDirty(); tmp->draw();
-  fb.addDirtyRect(x, y, w, h);
+  s.addDirtyRect(x, y, w, h);
 
   return tmp;
 }
@@ -300,11 +305,16 @@ void Widget::setDirtyInChain(Widget* start)
 StaticTextWidget::StaticTextWidget(GuiObject *boss, const GUI::Font& font,
                                    int x, int y, int w, int h,
                                    const string& text, TextAlignment align)
-    : Widget(boss, font, x, y, w, h),
-      _align(align)
+  : Widget(boss, font, x, y, w, h),
+    _align(align)
 {
   _flags = WIDGET_ENABLED | WIDGET_CLEARBG;
   _type = kStaticTextWidget;
+  _bgcolor = kDlgColor;
+  _bgcolorhi = kDlgColor;
+  _textcolor = kTextColor;
+  _textcolorhi = kTextColor;
+
   _label = text;
 }
 
@@ -328,9 +338,9 @@ void StaticTextWidget::setLabel(const string& label)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void StaticTextWidget::drawWidget(bool hilite)
 {
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
-  fb.drawString(_font, _label, _x, _y, _w,
-                isEnabled() ? _color : kColor, _align);
+  FBSurface& s = _boss->dialog().surface();
+  s.drawString(_font, _label, _x, _y, _w,
+               isEnabled() ? _textcolor : kColor, _align);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -339,11 +349,16 @@ ButtonWidget::ButtonWidget(GuiObject *boss, const GUI::Font& font,
                            const string& label, int cmd)
   : StaticTextWidget(boss, font, x, y, w, h, label, kTextAlignCenter),
     CommandSender(boss),
-    _cmd(cmd),
-    _editable(false)
+    _cmd(cmd)
 {
   _flags = WIDGET_ENABLED | WIDGET_BORDER | WIDGET_CLEARBG;
   _type = kButtonWidget;
+  _bgcolor = kBtnColor;
+  _bgcolorhi = kBtnColorHi;
+  _textcolor = kBtnTextColor;
+  _textcolorhi = kBtnTextColorHi;
+
+  _editable = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -390,14 +405,14 @@ void ButtonWidget::handleMouseUp(int x, int y, int button, int clickCount)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ButtonWidget::drawWidget(bool hilite)
 {
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
-  fb.drawString(_font, _label, _x, _y + (_h - _fontHeight)/2 + 1, _w,
-                !isEnabled() ? kColor : hilite ? kTextColorHi : _color, _align);
+  FBSurface& s = _boss->dialog().surface();
+  s.drawString(_font, _label, _x, _y + (_h - _fontHeight)/2 + 1, _w,
+               !isEnabled() ? kColor : hilite ? _textcolorhi : _textcolor, _align);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 /* 8x8 checkbox bitmap */
-static unsigned int checked_img[8] =
+static unsigned int checked_img_x[8] =
 {
 	0x00000000,
 	0x01000010,
@@ -409,13 +424,24 @@ static unsigned int checked_img[8] =
 	0x00000000,
 };
 
+static unsigned int checked_img_o[8] =
+{
+	0x00011000,
+	0x00111100,
+	0x01111110,
+	0x11111111,
+	0x11111111,
+	0x01111110,
+	0x00111100,
+	0x00011000,
+};
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CheckboxWidget::CheckboxWidget(GuiObject *boss, const GUI::Font& font,
                                int x, int y, const string& label,
                                int cmd)
   : ButtonWidget(boss, font, x, y, 16, 16, label, cmd),
     _state(false),
-    _editable(true),
     _holdFocus(true),
     _fillRect(false),
     _drawBox(true),
@@ -425,6 +451,9 @@ CheckboxWidget::CheckboxWidget(GuiObject *boss, const GUI::Font& font,
 {
   _flags = WIDGET_ENABLED;
   _type = kCheckboxWidget;
+  _bgcolor = _bgcolorhi = kWidColor;
+
+  _editable = true;
 
   if(label == "")
     _w = 14;
@@ -465,7 +494,6 @@ void CheckboxWidget::setState(bool state)
   if(_state != state)
   {
     _state = state;
-    _flags ^= WIDGET_INV_BORDER;
     setDirty(); draw();
   }
 }
@@ -473,28 +501,29 @@ void CheckboxWidget::setState(bool state)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckboxWidget::drawWidget(bool hilite)
 {
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
 
   // Draw the box
   if(_drawBox)
-    fb.box(_x, _y + _boxY, 14, 14, kColor, kShadowColor);
+    s.box(_x, _y + _boxY, 14, 14, kColor, kShadowColor);
 
-  // If checked, draw cross inside the box
-  if(_state)
+  // Do we draw a square or cross?
+  s.fillRect(_x + 2, _y + _boxY + 2, 10, 10, _bgcolor);
+  if(isEnabled())
   {
-    if(_fillRect)
-      fb.fillRect(_x + 2, _y + _boxY + 2, 10, 10,
-                  isEnabled() ? _color : kColor);
-    else
-      fb.drawBitmap(checked_img, _x + 3, _y + _boxY + 3,
-                    isEnabled() ? _color : kColor);
+    if(_state)
+    {
+      uInt32* img  = _fillRect ? checked_img_o : checked_img_x;
+	  uInt32 color = _fillRect ? kWidFrameColor : kCheckColor;
+	  s.drawBitmap(img, _x + 3, _y + _boxY + 3, color);
+    }
   }
   else
-    fb.fillRect(_x + 2, _y + _boxY + 2, 10, 10, kBGColor);
+    s.fillRect(_x + 2, _y + _boxY + 2, 10, 10, kColor);
 
   // Finally draw the label
-  fb.drawString(_font, _label, _x + 20, _y + _textY, _w,
-                isEnabled() ? _color : kColor);
+  s.drawString(_font, _label, _x + 20, _y + _textY, _w,
+               isEnabled() ? kTextColor : kColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -511,6 +540,8 @@ SliderWidget::SliderWidget(GuiObject *boss, const GUI::Font& font,
 {
   _flags = WIDGET_ENABLED | WIDGET_TRACK_MOUSE | WIDGET_CLEARBG;
   _type = kSliderWidget;
+  _bgcolor = kDlgColor;
+  _bgcolorhi = kDlgColor;
 
   if(!_label.empty() && _labelWidth == 0)
     _labelWidth = _font->getStringWidth(_label);
@@ -521,14 +552,8 @@ SliderWidget::SliderWidget(GuiObject *boss, const GUI::Font& font,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SliderWidget::setValue(int value)
 {
-/*cerr << "SliderWidget::setValue: " << value
-     << ", max = " << _valueMax
-     << ", min = " << _valueMin
-     << endl;*/
-  if(value < _valueMin)
-    value = _valueMin;
-  else if(value > _valueMax)
-    value = _valueMax;
+  if(value < _valueMin)      value = _valueMin;
+  else if(value > _valueMax) value = _valueMax;
 
   if(value != _value)
   {
@@ -542,14 +567,18 @@ void SliderWidget::setValue(int value)
 void SliderWidget::setMinValue(int value)
 {
   _valueMin = value;
-//  _stepValue = (int) ((_valueMax - _valueMin) * 0.05); // Step at 5% intervals
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SliderWidget::setMaxValue(int value)
 {
   _valueMax = value;
-//  _stepValue = (int) ((_valueMax - _valueMin) * 0.05); // Step at 5% intervals
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SliderWidget::setStepValue(int value)
+{
+  _stepValue = value;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -618,30 +647,39 @@ bool SliderWidget::handleEvent(Event::Type e)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SliderWidget::drawWidget(bool hilite)
 {
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
 
   // Draw the label, if any
   if(_labelWidth > 0)
-    fb.drawString(_font, _label, _x, _y + 2, _labelWidth,
-                  isEnabled() ? _color : kColor, kTextAlignRight);
+    s.drawString(_font, _label, _x, _y + 2, _labelWidth,
+                 isEnabled() ? kTextColor : kColor, kTextAlignRight);
 
   // Draw the box
-  fb.box(_x + _labelWidth, _y, _w - _labelWidth, _h, kColor, kShadowColor);
+  s.box(_x + _labelWidth, _y, _w - _labelWidth, _h, kColor, kShadowColor);
+
+  // Fill the box
+  s.fillRect(_x + _labelWidth + 2, _y + 2, _w - _labelWidth - 4, _h - 4,
+             !isEnabled() ? kColor : kWidColor);
 
   // Draw the 'bar'
-  fb.fillRect(_x + _labelWidth + 2, _y + 2, valueToPos(_value), _h - 4,
-              !isEnabled() ? kColor :
-              hilite ? kTextColorHi : _color);
+  s.fillRect(_x + _labelWidth + 2, _y + 2, valueToPos(_value), _h - 4,
+             !isEnabled() ? kColor : hilite ? kSliderColorHi : kSliderColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int SliderWidget::valueToPos(int value)
 {
+  if(value < _valueMin)      value = _valueMin;
+  else if(value > _valueMax) value = _valueMax;
+
   return ((_w - _labelWidth - 4) * (value - _valueMin) / (_valueMax - _valueMin));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int SliderWidget::posToValue(int pos)
 {
-  return (pos) * (_valueMax - _valueMin) / (_w - _labelWidth - 4) + _valueMin;
+  int value = (pos) * (_valueMax - _valueMin) / (_w - _labelWidth - 4) + _valueMin;
+
+  // Scale the position to the correct interval (according to step value)
+  return value - (value % _stepValue);
 }
