@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystemMACOSX.cxx,v 1.16 2007/01/03 12:59:23 stephena Exp $
+// $Id: OSystemMACOSX.cxx 1824 2009-07-01 16:04:28Z stephena $
 //============================================================================
 
 #include <cstdlib>
@@ -31,11 +31,6 @@
 #include "OSystemMACOSX.hxx"
 #include "MenusEvents.h"
 
-#ifdef HAVE_GETTIMEOFDAY
-  #include <time.h>
-  #include <sys/time.h>
-#endif
-
 extern "C" {
   void macOpenConsole(char *romname);
   uInt16 macOSXDisplayWidth(void);
@@ -56,25 +51,26 @@ extern char parentdir[MAXPATHLEN];
 //  the OS into the application.
 void macOpenConsole(char *romname)
 {
-	theOSystem->deleteConsole();
-	theOSystem->createConsole(romname);
+  theOSystem->deleteConsole();
+  theOSystem->createConsole(romname);
 }
 
 // Allow the Menus Objective-C object to pass event sends into the 
 //  application.
 void macOSXSendMenuEvent(int event)
 {
-    switch(event) {
-        case MENU_OPEN:
-            theOSystem->eventHandler().handleEvent(Event::LauncherMode, 1);
-            break;
-        case MENU_VOLUME_INCREASE:
-            theOSystem->eventHandler().handleEvent(Event::VolumeIncrease, 1);
-            break;
-        case MENU_VOLUME_DECREASE:
-            theOSystem->eventHandler().handleEvent(Event::VolumeDecrease, 1);
-            break;
-    }
+  switch(event)
+  {
+    case MENU_OPEN:
+      theOSystem->eventHandler().handleEvent(Event::LauncherMode, 1);
+      break;
+    case MENU_VOLUME_INCREASE:
+      theOSystem->eventHandler().handleEvent(Event::VolumeIncrease, 1);
+      break;
+    case MENU_VOLUME_DECREASE:
+      theOSystem->eventHandler().handleEvent(Event::VolumeDecrease, 1);
+      break;
+  }
 }
 
 /**
@@ -82,27 +78,19 @@ void macOSXSendMenuEvent(int event)
   in its constructor:
 
   setBaseDir()
-  setStateDir()
-  setPropertiesFiles()
   setConfigFile()
-  setCacheFile()
 
   See OSystem.hxx for a further explanation
 */
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-OSystemMACOSX::OSystemMACOSX(const string& path) : OSystem()
+OSystemMACOSX::OSystemMACOSX()
+  : OSystem()
 {
-  const string& basedir = (path.length() > 0) ? path :
-                           string(getenv("HOME")) + "/.stella";
-  setBaseDir(basedir);
+  setBaseDir("~/.stella");
 
-  setStateDir(basedir + "/state");
-
-  setPropertiesDir(basedir);
-  setConfigFile(basedir + "/stellarc");
-
-  setCacheFile(basedir + "/stella.cache");
+  // This will be overridden, as OSX uses plist files for settings
+  setConfigFile("~/.stella/stellarc");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -110,19 +98,6 @@ OSystemMACOSX::~OSystemMACOSX()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 OSystemMACOSX::getTicks()
-{
-#ifdef HAVE_GETTIMEOFDAY
-  timeval now;
-  gettimeofday(&now, 0);
-
-  return (uInt32) (now.tv_sec * 1000000 + now.tv_usec);
-#else
-  return (uInt32) SDL_GetTicks() * 1000;
-#endif
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystemMACOSX::getScreenDimensions(int& width, int& height)
 {
