@@ -8,18 +8,14 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsWin32.cxx,v 1.21 2006/01/14 21:36:29 stephena Exp $
+// $Id: SettingsWin32.cxx 1724 2009-05-13 13:55:40Z stephena $
 //============================================================================
 
-#include <sstream>
-#include <fstream>
-#include <direct.h>
-
 #include "bspf.hxx"
 #include "Settings.hxx"
 #include "SettingsWin32.hxx"
@@ -28,10 +24,11 @@
 SettingsWin32::SettingsWin32(OSystem* osystem)
   : Settings(osystem)
 {
-  set("fragsize", "2048");     // Anything less than this usually causes sound skipping
-  set("video", "hard");        // Use software mode with hardware surface
-  set("dirtyrects", "false");  // Most Windows systems work better without this
-  set("gl_lib", "opengl32.dll");
+  setInternal("gl_lib", "opengl32.dll");
+  // Anything less than this usually causes sound skipping
+  setInternal("fragsize", "2048");
+  // Most Windows systems work better without this
+  setInternal("romdir", "c:\\");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
