@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
 // Windows CE Port by Kostas Nakos
+// $Id: OSystemWinCE.cxx 1724 2009-05-13 13:55:40Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -25,23 +26,16 @@
 #include "SoundWinCE.hxx"
 #include "FrameBufferWinCE.hxx"
 
-extern void KeyCheck(void);
-extern int EventHandlerState;
-extern void KeySetMode(int);
 extern bool RequestRefresh;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 OSystemWinCE::OSystemWinCE(const string& path) : OSystem()
 {
-  string basedir = ((string) getcwd()) + '\\';
-  setBaseDir(basedir);
-
-  setStateDir(basedir);
-
-  setPropertiesDir(basedir);
-  setConfigFile(basedir + "stella.ini");
-
-  setCacheFile(basedir + "stella.cache");
+  setBaseDir(path);
+  setStateDir(path);
+  setPropertiesDir(path);
+  setConfigFile(path + "stella.ini");
+  setCacheFile(path + "stella.cache");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -62,19 +56,13 @@ void OSystemWinCE::mainLoop()
 
   uInt32 frameTime = 0, numberOfFrames = 0;
   uInt32 startTime, virtualTime, currentTime;
-  uInt8  lastkeyset;
 
   virtualTime = GetTickCount();
   frameTime = 0;
 
   // Main game loop
   MSG msg;
-  int laststate = -1;
-  if (!((FrameBufferWinCE *)myFrameBuffer)->IsSmartphoneLowRes())
-  {
-	  lastkeyset = 0;
-	  KeySetMode(1);
-  }
+
   for(;;)
   {
 	while (PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
@@ -87,11 +75,9 @@ void OSystemWinCE::mainLoop()
 	if (msg.message == WM_QUIT)
 		break;
 
-	if(myEventHandler->doQuit())
+	if (myQuitLoop)
 	  break;
 
-	KeyCheck();
-
 	if (RequestRefresh)
 	{
 		RequestRefresh = false;
@@ -100,29 +86,6 @@ void OSystemWinCE::mainLoop()
 
 	startTime = GetTickCount();
 
-	EventHandlerState = (int) myEventHandler->state();
-	if ((laststate != -1) && (laststate != EventHandlerState))
-		if (EventHandlerState!=2 && EventHandlerState!=3)
-		{
-			((FrameBufferWinCE *)myFrameBuffer)->wipescreen();
-			KeySetMode(lastkeyset);
-		}
-		else
-		{
-			if ( ((FrameBufferWinCE *)myFrameBuffer)->IsSmartphoneLowRes() )
-			{
-				KeySetMode(0);
-				((FrameBufferWinCE *)myFrameBuffer)->setmode(0);
-			}
-			else
-			{
-				lastkeyset = ((FrameBufferWinCE *)myFrameBuffer)->getmode();
-				if (lastkeyset == 0)
-					KeySetMode(1);
-			}
-		}
-	laststate = EventHandlerState;
-
 	myEventHandler->poll(startTime);
 	myFrameBuffer->update();
 	((SoundWinCE *)mySound)->update();
@@ -138,31 +101,31 @@ void OSystemWinCE::mainLoop()
 	frameTime += currentTime;
 	++numberOfFrames;
   }
-
-  /* {
-    double executionTime = (double) frameTime / 1000000.0;
-    double framesPerSecond = (double) numberOfFrames / executionTime;
-	ostringstream a;
-    a << endl;
-    a << numberOfFrames << " total frames drawn\n";
-    a << framesPerSecond << " frames/second\n";
-    a << endl;
-  TCHAR unicodeString[MAX_PATH];
-  MultiByteToWideChar(CP_ACP, 0, a.str().c_str(), strlen(a.str().c_str()) + 1, unicodeString, sizeof(unicodeString));
-	MessageBox(GetDesktopWindow(),unicodeString, _T("..."),0);
-   }
-*/
 }
 
-uInt32 OSystemWinCE::getTicks(void)
+uInt32 OSystemWinCE::getTicks(void) const
 {
 	return GetTickCount();
 }
 
-inline const GUI::Font& OSystemWinCE::launcherFont() const
+void OSystemWinCE::getScreenDimensions(int& width, int& height)
 {
-	if ( ((FrameBufferWinCE *)myFrameBuffer)->IsSmartphoneLowRes() )
-		return consoleFont();
+	// do not use the framebuffer issmartphonelowres method as the framebuffer has not been created yet
+	if ((unsigned int) GetSystemMetrics(SM_CXSCREEN) != 176 )
+	{
+		if (GetSystemMetrics(SM_CXSCREEN) != GetSystemMetrics(SM_CYSCREEN) )
+		{
+			width = 320;
+			height = 240;
+		}
+		else
+		{
+			width = height = 240;
+		}
+	}
 	else
-		return OSystem::font();
+	{
+		width = 220;
+		height = 176;
+	}
 }
