@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2006 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
 // Windows CE Port by Kostas Nakos
+// $Id: SettingsWinCE.cpp 1724 2009-05-13 13:55:40Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -27,7 +28,7 @@
 SettingsWinCE::SettingsWinCE(OSystem* osystem) : Settings(osystem) 
 {
   setInternal("romdir", (string) getcwd() + "\\Roms\\");
-  setInt("wce_smartphone_paddlespeed", 3000);
+  setInternal("wince_orientation", "0");
 }
 
 SettingsWinCE::~SettingsWinCE()
