@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2006 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
 // Windows CE Port by Kostas Nakos
+// $Id: missing.h 1724 2009-05-13 13:55:40Z stephena $
 //============================================================================
 
 #ifndef _WCE_MISSING_H_
@@ -40,9 +41,4 @@ typedef unsigned int  uintptr_t;
 int time(int dummy);
 char *getcwd(void);
 
-
-#define MAX_KEYS 8
-#define NUM_MOUSEKEYS 2
-enum key {K_UP = 0, K_DOWN, K_LEFT, K_RIGHT, K_FIRE, K_RESET, K_SELECT, K_QUIT, M_POS, M_BUT};
-
 #endif
